"""
Compatibility shim for legacy imports.
Re-exports configuration loading utilities from pyegeria.config.
"""
# Import config lazily inside functions to avoid import-time side effects in tests
from importlib import import_module

def load_app_config(env_file: str | None = None):
    return import_module('pyegeria.config').load_app_config(env_file)

def get_app_config(env_file: str | None = None):
    return import_module('pyegeria.config').get_app_config(env_file)

class PyegeriaSettings:
    def __new__(cls, *args, **kwargs):
        # Construct a real settings instance from pyegeria.config
        return import_module('pyegeria.config').PyegeriaSettings(*args, **kwargs)

    @classmethod
    def with_env_file(cls, env_file: str):
        return import_module('pyegeria.config').PyegeriaSettings.with_env_file(env_file)

# expose the cached instance for tests that reset it directly
from . import config as _config

def __getattr__(name):
    if name == "_app_config":
        return _config._app_config
    raise AttributeError(name)


def __setattr__(name, value):
    if name == "_app_config":
        _config._app_config = value
    else:
        globals()[name] = value



#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

Unit tests for the Utils helper functions using the Pytest framework.


A simple status display of a user's todos
"""
import argparse
import os
import time

from rich import box
from rich.console import Console
from rich.live import Live
from rich.markdown import Markdown
from rich.table import Table

from pyegeria import (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
)
from pyegeria.my_profile_omvs import MyProfile

EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get("KAFKA_ENDPOINT", "localhost:9092")
EGERIA_PLATFORM_URL = os.environ.get("EGERIA_PLATFORM_URL", "https://localhost:9443")
EGERIA_VIEW_SERVER = os.environ.get("EGERIA_VIEW_SERVER", "view-server")
EGERIA_VIEW_SERVER_URL = os.environ.get(
    "EGERIA_VIEW_SERVER_URL", "https://localhost:9443"
)
EGERIA_INTEGRATION_DAEMON = os.environ.get("EGERIA_INTEGRATION_DAEMON", "integration-daemon")
EGERIA_INTEGRATION_DAEMON_URL = os.environ.get(
    "EGERIA_INTEGRATION_DAEMON_URL", "https://localhost:9443"
)
EGERIA_ADMIN_USER = os.environ.get("ADMIN_USER", "garygeeke")
EGERIA_ADMIN_PASSWORD = os.environ.get("ADMIN_PASSWORD", "secret")
EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")
EGERIA_JUPYTER = bool(os.environ.get("EGERIA_JUPYTER", "False"))
EGERIA_WIDTH = int(os.environ.get("EGERIA_WIDTH", "200"))

disable_ssl_warnings = True


def display_my_todos(
    server: str,
    url: str,
    user: str,
    user_pass: str,
    jupyter: bool = EGERIA_JUPYTER,
    width: int = EGERIA_WIDTH,
):
    console = Console(width=width, force_terminal=not jupyter)

    m_client = MyProfile(server, url, user_id=user)
    token = m_client.create_egeria_bearer_token(user, user_pass)

    def add_rows(table: Table, guid: str, identity: str) -> None:
        todo_items = m_client.get_assigned_actions(guid)
        if type(todo_items) is str:
            return

        for item in todo_items:
            assigned_actors = ""
            if todo_items is None:
                name = " "
                type_name = " "
                todo_guid = " "
                created = " "
                priority = " "
                due = " "
                completed = " "
                status = " "
                sponsor = " "
            else:
                props = item["properties"]
                name = props["name"]
                todo_type_name = props.get("toDoType", " ")
                todo_guid = item["elementHeader"].get("guid", "---")
                created = props.get("creationTime", "---")
                priority = str(props.get("priority", "---"))
                due = props.get("dueTime", " ")
                completed = props.get("completionTime", "---")
                status = props.get("toDoStatus")

                for actor in item["assignedActors"]:
                    assigned_actors += f"{actor.get("uniqueName", "NoOne")}\n"
                assigned_actors_out = Markdown(assigned_actors)
                if status in ("WAITING", "OPEN"):
                    status = f"[yellow]{status}"
                elif status in ("INPROGRESS", "COMPLETE"):
                    status = f"[green]{status}"
                else:
                    status = f"[red]{status}"

            table.add_row(
                str(identity),
                name,
                todo_type_name,
                todo_guid,
                created,
                priority,
                due,
                completed,
                status,
                assigned_actors_out,
            )

    def generate_table() -> Table:
        """Make a new table."""
        table = Table(
            title=f"Open ToDos for Platform {url} @ {time.asctime()}",
            # style = "black on grey66",
            header_style="white on dark_blue",
            show_lines=True,
            box=box.ROUNDED,
            caption=f"ToDos for Server '{server}' @ Platform - {url}",
            expand=True,
        )
        table.add_column("Actor")
        table.add_column("ToDo Name")
        table.add_column("Type Name")
        table.add_column("GUID")
        table.add_column("Created")
        table.add_column("Priority")
        table.add_column("Due")
        table.add_column("Completion")
        table.add_column("Status")
        table.add_column("Sponsor")

        my_profile = m_client.get_my_profile()
        my_guid = my_profile["elementHeader"].get("guid", "---")
        my_ids = my_profile["userIdentities"]
        my_title = my_profile["profileProperties"].get("jobTitle", "No Title")
        user_ids = []
        for id in my_ids:
            user_ids.append(id["userIdentity"]["properties"].get("userid", "NoOne"))
            add_rows(
                table,
                id["userIdentity"]["elementHeader"]["guid"],
                id["userIdentity"]["properties"]["userId"],
            )

        add_rows(table, my_guid, user_ids)

        # user_id_guid = my_profile["userIdentity"]["guid"]
        # user_id_name = my_profile["userIdentity"]["properties"]["userid"]
        # add_rows(table, user_id_guid, user_id_name)

        my_roles = my_profile["roles"]
        if type(my_roles) is list:
            for role in my_roles:
                role_guid = role["elementHeader"]["guid"]
                role_id = role["properties"].get("roleId", "No Role Id")
                add_rows(table, role_guid, role_id)

        token = m_client.refresh_egeria_bearer_token()
        return table

    try:
        with Live(generate_table(), refresh_per_second=4, screen=True) as live:
            while True:
                time.sleep(2)
                live.update(generate_table())

    except (
        InvalidParameterException,
        PropertyServerException,
        UserNotAuthorizedException,
    ) as e:
        print_exception_response(e)

    except KeyboardInterrupt:
        pass
    finally:
        m_client.close_session()


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--server", help="Name of the view server to connect to")
    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")
    args = parser.parse_args()

    server = args.server if args.server is not None else EGERIA_VIEW_SERVER
    url = args.url if args.url is not None else EGERIA_PLATFORM_URL
    userid = args.userid if args.userid is not None else "peterprofile"
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD

    try:
        print(f"Starting display_todos with {server}, {url}, {userid}")
        display_my_todos(server=server, url=url, user=userid, user_pass=user_pass)
    except KeyboardInterrupt:
        pass


if __name__ == "__main__":
    main()



"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.



This module is for testing the data designer view service class and methods.
The routines assume that pytest is being used as the test tool and framework.

A running Egeria environment is needed to run these tests.

"""

import json
import time, os

from rich import print, print_json
from rich.console import Console

from pyegeria import PyegeriaAPIException
from pyegeria._deprecated_gov_engine import body_slimmer
from pyegeria._exceptions import (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
    )
from pyegeria._exceptions_new import PyegeriaException, print_basic_exception, print_exception_table
from pyegeria.data_designer import DataDesigner
from pyegeria.config import settings

disable_ssl_warnings = True
PLATFORM_URL = settings.Environment.egeria_platform_url
VIEW_SERVER = settings.Environment.egeria_view_server
USER_PWD = os.environ.get("EGERIA_USER_PASSWORD", "secret")


console = Console(width=250)


def jprint(info, comment=None):
    if comment:
        print(comment)
    print(json.dumps(info, indent=2))


def valid_guid(guid):
    if (guid is None) or (type(guid) is not str):
        return False
    else:
        return True


EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")
USER_PWD = EGERIA_USER_PASSWORD


import pytest



class TestDataDesigner:
    platform_url = PLATFORM_URL
    view_server = VIEW_SERVER
    user = EGERIA_USER
    password = USER_PWD

    #
    ##
    #


    def test_create_data_structure(self):
        display_name = "solar_power2"
        namespace = "solar"
        # body = {
        #     "class": "NewElementRequestBody",
        #     "isOwnAnchor": True,
        #     "properties": {
        #         "class" : "DataStructureProperties",
        #         "displayName": display_name,
        #         "qualifiedName": f"{namespace}::data-structure::{display_name}",
        #         "description": "a solar power data structure",
        #         "namespace": namespace,
        #         "versionIdentifier": "0.1"
        #       }
        #     }
        body = {

          "class" : "NewElementRequestBody",
          "isOwnAnchor" : True,
          "initialClassifications" : { },
          "initialStatus" : "ACTIVE",
          "parentAtEnd1" : True,
          "properties" : {
            "class" : "DataSpecProperties",
            "displayName" : "Data Specification for the Teddy Bear Drop Foot Clinical Trial",
            "qualifiedName" : "DataSpec::Data Specification for the Teddy Bear Drop Foot Clinical Trial",
            "description" : "Principle data requirements for this clinical trial."

          }
        }
        body = body_slimmer(body)
        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            response = m_client.create_data_structure(body)
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds, Type: {type(response)}"
            )
            if type(response) is list:
                print_json(data=response)
            elif type(response) is str:
                console.print("\n\n\t Response is: " + response)

            assert True
        except (
            PyegeriaException
        ) as e:
            print_basic_exception(e)
            # assert False, "Invalid request"

        finally:
            m_client.close_session()

    def test_delete_data_structure(self):
        guid = 'ed7e21b3-6c94-48dd-b24d-a0ce923754ba'
        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            m_client.delete_data_structure(guid)
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds"
            )

            assert True
        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()

    def test_link_member_data_field(self):
        struct_guid = "e441f9f1-4dd0-4960-b11d-08a1b8c717f6"
        member_guid = "ca085aa8-c5bb-4a15-ab2b-73be43166f3c"

        body = {
          "class" : "MemberDataFieldRequestBody",
          "properties": {
            "class": "MemberDataFieldProperties",
            "dataFieldPosition": 0,
            "minCardinality": 0,
            "maxCardinality": -1,

          }
        }

        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            m_client.link_member_data_field(struct_guid,member_guid, body)
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds"
            )

            assert True
        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()

    def test_link_data_class_definition(self):
        data_field_guid = "2ca5f6c9-1c9e-40b7-bffd-0ec440c1673d"
        data_class_guid = "7f2d5160-0133-4f23-b3fe-3c3d6f5a2be6"

        body = {
            "class": "MetadataSourceRequestBody",
            "forLineage": False,
            "forDuplicateProcessing": False
            }

        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            m_client.link_data_class_definition(data_field_guid,data_class_guid, body)
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds"
            )

            assert True
        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()




    def test_find_all_data_structures(self):
        columns_struct = {
            "heading": "data structs",
            "description": "structs generic to all Agreements.",
            "aliases": [],

            "formats": [{'types': ["ALL"],
                "attributes": [
                    {'name': 'Name', 'key': 'display_name'},
                    {'name': 'Qualified Name', 'key': 'qualified_name', 'format': True},
                    {'name': 'Super Category', 'key': 'category'},
                    {'name': 'My Description', 'key': 'description', 'format': True},
                    {'name': "Classifications", 'key': 'classifications'},
                    {'name': 'Members', 'key': 'members', 'format': True},
                    {'name': 'Mermaid', 'key': 'mermaid'},
                    ],
                         }
                        ],
            "annotations": {"wikilinks": ["[[Egeria]]"]}
            }
        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            response = m_client.find_all_data_structures(output_format="REPORT", report_spec = "Data Structures")
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds, Type: {type(response)}"
                )
            if type(response) is list:
                print_json(data=response)
            elif type(response) is str:
                console.print("\n\n\t Response is: " + response)

            assert True
        except (
                PyegeriaException
                ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()

    def test_find_data_structures(self):
        report_spec = {
            "heading": "data structs",
            "description": "structs generic to all Agreements.",
            "formats": [{"types": "ALL",
                "attributes": [
                    {'name': 'Name', 'key': 'display_name'},
                    {'name': 'Qualified Name', 'key': 'qualified_name', 'format': True},
                    {'name': 'Super Category', 'key': 'category'},
                    {'name': 'My Description', 'key': 'description', 'format': True},
                    {'name': "Classifications", 'key': 'classifications'},
                    {'name': 'Members', 'key': 'containsDataFields', 'format': True},
                    {'name': 'CreatedBy Meow', 'key': 'created_by'}
                    ]}
                ],
            "annotations": {"wikilinks": ["[[Egeria]]"]}
            }
        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            search_string = "*"
            response = m_client.find_data_structures(search_string, output_format="DICT",
                                                     report_spec=report_spec)
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds, Type: {type(response)}"
                )
            if type(response) is list:
                print_json(data=response)
            elif type(response) is str:
                console.print("\n\n\t Response is: " + response)

            assert True
        except (
                PyegeriaException
                ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()

    def test_get_data_struct_coll(self):

        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            guid = "feb88a1f-2e5d-4cdd-ae1d-36613d311ea8"
            response = m_client.get_data_memberships(m_client.get_data_structure_by_guid, guid)
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds, Type: {type(response)}"
                )
            if type(response) is dict:
                print_json(data=response)
            elif type(response) is str:
                console.print("\n\n\t Response is: " + response)

            assert True
        except (
                PyegeriaException,
                ) as e:
            print_basic_exception(e)
            # assert False, "Invalid request"

        finally:
            m_client.close_session()


    def test_get_data_structures_by_name(self):
        name = "DataStruct::TBDF-Incoming Weekly Measurement Data"
        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            response = m_client.get_data_structures_by_name(name, output_format="REPORT")
            duration = time.perf_counter() - start_time
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds, Type: {type(response)}"
                )
            if type(response) is list:
                print_json(data=response)
            elif type(response) is str:
                console.print("\n\n\t Response is: " + response)

            assert True
        except (
                PyegeriaException
                ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()

    def test_get_data_structures_by_guid(self):
        guid = "dc13d802-3b91-42d9-99e0-3d31f0bb2dd8"
        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            response = m_client.get_data_structure_by_guid(guid, output_format="REPORT", report_spec="Mandy-DataStruct")
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds, Type: {type(response)}"
                )
            if isinstance(response, list | dict):
                print_json(data=response)
            elif type(response) is str:
                console.print("\n\n\tResponse is: \n" + response)

            assert True
        except (
                PyegeriaException
                ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()


    def test_create_data_field(self):
        name = "Radio Name"
        description = "Test Data Field Description"

        body =  {
            "class": "NewElementRequestBody",
            "forLineage": False,
            "forDuplicateProcessing": False,
            "isOwnAnchor": True,
            "properties": {
                "class": "DataFieldProperties",
                "qualifiedName": "dataField::radio_name",
                "displayName": "radio_name",
                "namespace": "",
                "description": "What is the name of the radio",
                "versionIdentifier": ".1",
                "aliases": [
                  "transmitter"
                ],
                "namePatterns": [],
                "isDeprecated": False,
                "isNullable": False,
                "dataType": "String",
                "minimumLength": 3,
                "length": 20,
                "precision": 0,
                "orderedValues": False,
                "sortOrder": "UNSORTED",

            }
          }



        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            response = m_client.create_data_field(body)
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds, Type: {type(response)}"
            )
            if type(response) is list:
                print_json(data=response)
            elif type(response) is str:
                console.print("\n\n\t Response is: " + response)

            assert True
        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()

    def test_find_all_data_fields(self):

        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            response = m_client.find_all_data_fields(output_format="REPORT", report_spec="DataFields")
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds, Type: {type(response)}"
                )
            if type(response) is list:
                print_json(data=response)
            elif type(response) is str:
                console.print("\n\n\t Response is: \n\n" + response)

            assert True
        except (
                PyegeriaException
                ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()

    def test_get_data_field_by_name(self):
        name = "AngleRight"
        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            response = m_client.get_data_fields_by_name(filter_string=name, output_format="DICT", report_spec="DataField-DrE")
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds, Type: {type(response)}"
                )
            if isinstance (response, list | dict):
                print_json(data=response)
            elif type(response) is str:
                console.print("\n\n\t Response is: " + response)

            assert True
        except (
                PyegeriaException
                ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()

    def test_get_data_field_by_guid(self):
        guid = '834698cd-43e0-4516-858d-9020e89e82c7'
        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            response = m_client.get_data_field_by_guid(guid, output_format="JSON", report_spec="DataField-DrE")
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds, Type: {type(response)}"
                )
            if isinstance(response, list | dict):
                print_json(data=response)
            elif type(response) is str:
                console.print("\n\n\t Response is: " + response)

            assert True
        except (
                PyegeriaException
                ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()

    def test_delete_data_field(self):
        guid = 'b8c82425-7c69-4311-a3e0-86d0efd5cb30'
        # guid = 'b18df5d5-23c6-4c85-a06d-e1da6088901c'

        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            m_client.delete_data_field(guid)
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds"
            )

            assert True
        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()

    def test_get_data_field_rel_elements(self):
        guid = "834698cd-43e0-4516-858d-9020e89e82c7"
        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            response = m_client.get_data_field_rel_elements(guid)
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds, Type: {type(response)}"
                )
            if isinstance(response, list | dict):
                print_json(data=response)
            elif type(response) is str:
                console.print("\n\n\t Response is: " + response)

            assert True
        except (
                PyegeriaException
                ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()

    def test_link_nested_data_field(self):
        parent_guid = "a16cb2dd-22cd-4ffe-bca0-b48ac1b1762d"
        child_guid = "407e0012-8a13-444a-8f96-e74ad77ed55c"
        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            response = m_client.link_nested_data_field(parent_guid, child_guid)
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds, Type: {type(response)}"
                )
            if isinstance(response, list | dict):
                print_json(data=response)
            elif type(response) is str:
                console.print("\n\n\t Response is: " + response)

            assert True
        except (
                PyegeriaException
                ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()



    def test_find_data_fields(self):

        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            search_string = ''
            response = m_client.find_data_fields(search_string,output_format="REPORT")
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds, Type: {type(response)}"
                )
            if type(response) is list:
                print_json(data=response)
            elif type(response) is str:
                console.print("\n\n\t Response is: " + response)

            assert True
        except (
                PyegeriaException
                ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()


    def test_find_data_fields_w_body(self):

        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            search_string = "*"

            body = {
                "class": "FilterRequestBody",
                # "asOfTime": "2025-06-27T16:10:00-05:00",
                "effectiveTime": None,
                "forLineage": False,
                "forDuplicateProcessing": False,
                "limitResultsByStatus": ["ACTIVE"],
                "sequencingOrder": None,
                "sequencingProperty": None,
                "filter": search_string,
                }
            response = m_client.find_data_fields_w_body(body,output_format="DICT")
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds, Type: {type(response)}"
                )
            if type(response) is list:
                print_json(data=response)
            elif type(response) is str:
                console.print("\n\n\t Response is: " + response)

            assert True
        except (
                PyegeriaException
                ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()





#
# Data Classes
#
    def test_create_data_class(self):
        name = "Purchase Date"
        description = "Date of purchase in YYYY-MM-DD format"

        body = {
          "class" : "NewElementRequestBody",

          "effectiveTime" : None,
          "forLineage" : False,
          "forDuplicateProcessing" : False,
          # "anchorGUID" : "add guid here",
          "isOwnAnchor": True,
          # "parentGUID": "add guid here",
          # "parentRelationshipTypeName": "add type name here",
          # "parentRelationshipProperties": {
          #   "class": "ElementProperties",
          #   "propertyValueMap" : {
          #     "description" : {
          #       "class": "PrimitiveTypePropertyValue",
          #       "typeName": "string",
          #       "primitiveValue" : "New description"
          #     }
          #   }
          # },
          "parentAtEnd1": True,
          "properties": {
            "class" : "DataClassProperties",
            "qualifiedName": "dataClass::moo",
            "displayName": "Moo",
            "description": "add description here",
            # "namespace": "add scope of this data class's applicability.",
            # "matchPropertyNames": ["name1", "name2"],
            "matchThreshold": 0,
            # "specification": "",
            # "specificationDetails": {
            #   "property1" : "propertyValue1",
            #   "property2" : "propertyValue2"
            # },
            "dataType": "String",
            "allowsDuplicateValues": True,
            "isNullable": False,
            "defaultValue": "",
            "averageValue": "",
            "valueList": [],
            "valueRangeFrom": "",
            "valueRangeTo": "",
            "sampleValues": [],
            "dataPatterns" : [],
            "namePatterns": [],
            # "additionalProperties": {
            #   "property1" : "propertyValue1",
            #   "property2" : "propertyValue2"
            # },
            # "effectiveFrom": "{{$isoTimestamp}}",
            # "effectiveTo": "{{$isoTimestamp}}"
          }
        }

            # {
            # "properties": {
            #     "class": "DataClassProperties",
            #     "qualifiedName": "DataClass::Purchase Date",
            #     "displayName": name,
            #     "description": description,
            #     "namespace": None,
            #     "matchPropertyNames": [],
            #     "matchThreshold": 0,
            #     "specification": None,
            #     "specificationDetails": {},
            #     "dataType": "Date",
            #     "allowsDuplicateValues": True,
            #     "isNullable": False,
            #     "defaultValue": None,
            #     "averageValue": None,
            #     "valueList": [],
            #     "valueRangeFrom": "",
            #     "valueRangeTo": "",
            #     "sampleValues": [],
            #     "dataPatterns": [],
            #     "additionalProperties": { }
            #     }
            # }

        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            response = m_client.create_data_class(body)
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds, Type: {type(response)}"
                )
            if type(response) is list:
                print_json(data=response)
            elif type(response) is str:
                console.print("\n\n\t Response is: " + response)

            assert True
        except (
                PyegeriaException
                ) as e:
            print_basic_exception(e)
            # assert False, "Invalid request"

        finally:
            m_client.close_session()

    def test_find_all_data_classes(self):

        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            response = m_client.find_all_data_classes(output_format="DICT", report_spec="DataClass-DrE")
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds, Type: {type(response)}"
                )
            if type(response) is list:
                print_json(data=response)
            elif type(response) is str:
                console.print("\n\n\t Response is: " + response)

            assert True
        except (
                PyegeriaException
                ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()

    def test_find_data_classes(self):

        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            # search_string = "DataClass::ISO-Date"
            search_string = "*"
            start_time = time.perf_counter()
            response = m_client.find_data_classes(search_string,output_format="DICT", report_spec="DataClass-DrE")
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds, Type: {type(response)}"
                )
            if type(response) is list:
                print_json(data=response)
            elif type(response) is str:
                console.print("\n\n\t Response is: " + response)
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds, Type: {type(response)}"
                )

            if type(response) is list:
                print_json(data=response)
            elif type(response) is str:
                console.print("\n\n\t Response is: " + response)

            assert True
        except (
                PyegeriaException
                ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()

    def test_get_data_class_by_name(self):
        name = "DataClass::ISO-Date"
        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            response = m_client.get_data_classes_by_name(name, output_format="JSON")
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds, Type: {type(response)}"
                )
            if type(response) is list:
                print_json(data=response)
            elif type(response) is str:
                console.print("\n\n\t Response is: " + response)

            assert True
        except (
                PyegeriaException
                ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()

    def test_get_data_class_by_guid(self):
        guid = '6af8cfc5-3218-4abc-a174-4c852856765e'
        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            response = m_client.get_data_class_by_guid(guid, output_format="DICT")
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds, Type: {type(response)}"
                )
            if isinstance(response, list | dict):
                print_json(data=response)
            elif type(response) is str:
                console.print("\n\n\t Response is: " + response)

            assert True
        except (
                PyegeriaException
                ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()

    def test_link_nested_data_class(self):
        containing_guid = "e441f9f1-4dd0-4960-b11d-08a1b8c717f6"
        member_guid = "ca085aa8-c5bb-4a15-ab2b-73be43166f3c"

        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            m_client.link_nested_data_class(containing_guid, member_guid)
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds"
                )

            assert True
        except (
                PyegeriaException
                ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()

    def test_link_specialist_data_class(self):
        containing_guid = "6af8cfc5-3218-4abc-a174-4c852856765e"
        member_guid = "198ca7cf-76d5-4504-8a94-c1158c23cc7a"

        try:
            m_client = DataDesigner(self.view_server, self.platform_url)

            m_client.create_egeria_bearer_token(self.user, self.password)
            start_time = time.perf_counter()
            m_client.link_specialist_data_class(containing_guid, member_guid)
            duration = time.perf_counter() - start_time
            print(
                f"\n\tDuration was {duration:.2f} seconds"
                )

            assert True
        except (
                PyegeriaException
                ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            m_client.close_session()


"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.



This module is for testing the glossary manager class and methods
The routines assume that pytest is being used as the test tool and framework.

A running Egeria environment is needed to run these tests.

"""

import asyncio
import json
import time
from contextlib import nullcontext as does_not_raise

import pytest
from pydantic import ValidationError

from pyegeria import PyegeriaException, print_exception_table, print_validation_error, \
    PyegeriaInvalidParameterException, print_basic_exception
from pyegeria._exceptions import (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
)
from pyegeria.glossary_manager import GlossaryManager, GlossaryTermProperties
from pyegeria.core_omag_server_config import CoreServerConfig
from pyegeria.models import NewElementRequestBody
from tests.test_classification_manager_omvs import relationship_type
from tests.test_feedback_manager_omvs import password

# from pyegeria.admin_services import FullServerConfig

disable_ssl_warnings = True


class TestGlossaryManager:
    good_platform1_url = "https://127.0.0.1:9443"
    good_platform2_url = "https://oak.local:9443"
    bad_platform1_url = "https://localhost:9443"

    good_user_1 = "garygeeke"
    good_user_2 = "erinoverview"
    good_user_3 = "peterprofile"
    bad_user_1 = "eviledna"
    bad_user_2 = ""
    good_user_2_pwd = "secret"
    good_server_1 = "simple-metadata-store"
    good_server_2 = "qs-view-server"
    good_server_3 = "active-metadata-store"
    good_server_4 = "integration-daemon"
    good_server_5 = "fluffy_kv"
    good_server_6 = "cocoVIew1"
    good_engine_host_1 = "governDL01"
    good_view_server_1 = "view-server"
    good_view_server_2 = "qs-view-server"
    bad_server_1 = "coco"
    bad_server_2 = ""

    def test_create_glossary(self):
        try:
            g_client = GlossaryManager(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd=self.good_user_2_pwd,
            )

            token = g_client.create_egeria_bearer_token(
                self.good_user_2, self.good_user_2_pwd
            )
            start_time = time.perf_counter()
            display_name = "test"
            description = "A glossary used for test"
            language = "English"
            usage = "for testing purposes only"
            response = g_client.create_glossary(
                display_name, description, language, usage
            )
            duration = time.perf_counter() - start_time
            # resp_str = json.loads(response)
            print(f"\n\tDuration was {duration} seconds")
            print(f"\n\nNew glossary {display_name} created with GUID of {response}")
            assert True
        except (
                PyegeriaException
        ) as e:
            print_exception_table(e)
            assert False, "Invalid request"
        except ValidationError as e:
            print_validation_error(e)
        finally:
            g_client.close_session()

    def test_update_glossary(self):
        try:
            g_client = GlossaryManager(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd=self.good_user_2_pwd,
            )

            token = g_client.create_egeria_bearer_token(
                self.good_user_2, self.good_user_2_pwd
            )
            glossary_guid = "b336de33-ae66-4db9-b669-85c146c4b053"
            start_time = time.perf_counter()
            body = {
                "class": "UpdateElementRequestBody",
                "properties": {
                    "class": "GlossaryProperties",
                    "displayName": "puddys-universe",
                    "qualified_name": "Glossary:puddys-universe",
                },
            }

            g_client.update_glossary(glossary_guid, body)
            duration = time.perf_counter() - start_time
            # resp_str = json.loads(response)
            print(f"\n\tDuration was {duration} seconds")
            print(f"\n\nUpdated glossary {glossary_guid}")
            assert True
        except (
                    PyegeriaException
        ) as e:
            print_exception_table(e)
            assert False, "Invalid request"
        except ValidationError as e:
            print_validation_error(e)
        finally:
            g_client.close_session()

    def test_delete_glossary(self):
        try:
            g_client = GlossaryManager(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd=self.good_user_2_pwd,
            )

            token = g_client.create_egeria_bearer_token(
                self.good_user_2, self.good_user_2_pwd
            )
            start_time = time.perf_counter()
            glossary_guid = "46ec84bc-7264-4cf4-86bc-253cca63e115"
            g_client.delete_glossary(glossary_guid, cascade = True)
            duration = time.perf_counter() - start_time

            print(f"\n\tDuration was {duration} seconds")
            print(f"\n\nDeleted glossary {glossary_guid}")
            assert True
        except (
                    PyegeriaException
        ) as e:
            print_exception_table(e)
            assert False, "Invalid request"
        finally:
            g_client.close_session()



    def test_create_term(self):
        try:
            g_client = GlossaryManager(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd=self.good_user_2_pwd,
            )

            token = g_client.create_egeria_bearer_token(
                self.good_user_2, self.good_user_2_pwd
            )
            start_time = time.perf_counter()
            glossary_guid ="6222f556-9123-4bb2-ba60-d2fe2b1d7fe9"
            qualified_name = "GlossaryTerm:test-term"
            prop_body = GlossaryTermProperties(class_ = "GlossaryTermProperties",
                                               display_name = "test-term",
                                                description = "A test term",
                                                qualified_name = qualified_name,
                                               summary = "a quick summary",
                                               abbreviation = "dt2",
                                               examples="an example",
                                               usage="an usage example",
                                               )

            body = NewElementRequestBody(class_ = "NewElementRequestBody",
                                         parent_guid= glossary_guid,
                                        is_own_anchor= True,
                                        anchor_scope_guid= glossary_guid,
                                        parent_relationship_type_name = "CollectionMembership",
                                        parent_at_end_1 = True,
                                        properties = prop_body.model_dump(exclude_none=True),
                                         )

            response = g_client.create_glossary_term(body)

            duration = time.perf_counter() - start_time

            print(f"\n\tDuration was {duration} seconds")

            assert True
        except (
                    PyegeriaException
        ) as e:
            print_exception_table(e)
            assert False, "Invalid request"
        except ValidationError as e:
            print_validation_error(e)
        finally:
            g_client.close_session()

    def test_create_term_copy(self):
        try:
            g_client = GlossaryManager(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd=self.good_user_2_pwd,
            )

            token = g_client.create_egeria_bearer_token(
                self.good_user_2, self.good_user_2_pwd
            )
            start_time = time.perf_counter()
            glossary_guid ="9bf9e61a-bc4f-4e52-ac2a-c5c46eda950e"
            term_guid = "e6bfae8b-07ad-44be-a4a4-0a4310cb8964"

            response = g_client.create_term_copy(glossary_guid, term_guid, "new-test1")

            duration = time.perf_counter() - start_time
            print(f"Response is: {response}")
            print(f"\n\tDuration was {duration} seconds")

            assert True
        except (
                    PyegeriaException
        ) as e:
            print_exception_table(e)
            assert False, "Invalid request"
        except ValidationError as e:
            print_validation_error(e)
        finally:
            g_client.close_session()




    def test_add_is_abstract_concept(self):
        try:
            g_client = GlossaryManager(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd=self.good_user_2_pwd,
            )

            token = g_client.create_egeria_bearer_token(
                self.good_user_2, self.good_user_2_pwd
            )


            start_time = time.perf_counter()
            category_guid = "6d848f56-0332-4c8f-a048-9209d912809b"
            term_guid =  "a6247ae4-0606-4aa9-94e1-4f0b0713cc58"
            g_client.add_is_abstract_concept(term_guid)
            duration = time.perf_counter() - start_time

            print(f"\n\tDuration was {duration} seconds")
            assert True
        except  (PyegeriaException, PyegeriaInvalidParameterException) as e:
            print_exception_table(e)
            assert False, "Invalid request"
        except ValidationError as e:
            print_validation_error(e)
        finally:
            g_client.close_session()


    def test_find_glossaries(self):
        try:
            g_client = GlossaryManager(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
            )

            token = g_client.create_egeria_bearer_token(self.good_user_2, "secret")
            start_time = time.perf_counter()
            response = g_client.find_glossaries(
                "*",
                starts_with=False,
                ends_with=False,
                ignore_case=True,
                page_size=0,
                output_format="JSON"
            )
            duration = time.perf_counter() - start_time
            # resp_str = json.loads(response)
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is list:
                # print("\n\n" + json.dumps(response, indent=4))
                count = len(response)
                print(f"Found {count} glossaries")

                print(json.dumps(response, indent=4))
            elif type(response) is str:
                print("\n\n" + response)
            assert True

        except (
                    PyegeriaException
        ) as e:
            print_exception_table(e)
            assert False, "Invalid request"
        finally:
            g_client.close_session()

    def test_get_glossary_by_guid(self, server: str = good_view_server_2):
        try:
            server_name = server
            g_client = GlossaryManager(
                server_name, self.good_platform1_url, user_id=self.good_user_2
            )

            token = g_client.create_egeria_bearer_token(self.good_user_2, "secret")
            # glossary_guid = "f9b78b26-6025-43fa-9299-a905cc6d1575"  # This is the sustainability glossary
            # glossary_guid = (
            #     "ab84bad2-67f0-4ec8-b0e3-76e638ec9f63"  # This is CIM glossary
            # )
            glossary_guid = '8f829009-376e-42ae-9f5d-32433755ff92'
            response = g_client.get_glossary_by_guid(glossary_guid)
            print(f"type is {type(response)}")
            if type(response) is dict:
                print("\n\n" + json.dumps(response, indent=4))
            elif type(response) is str:
                print("\n\n" + response)
            assert True
        except (
                    PyegeriaException
        ) as e:
            print_exception_table(e)
            assert False, "Invalid request"

        finally:
            g_client.close_session()

    def test_get_glossaries_by_name(self, server: str = good_view_server_2):
        try:
            server_name = server
            g_client = GlossaryManager(
                server_name, self.good_platform1_url, user_id=self.good_user_2
            )

            token = g_client.create_egeria_bearer_token(self.good_user_2, "secret")
            glossary_name = "Egeria-Markdown"

            response = g_client.get_glossaries_by_name(glossary_name)
            print(f"type is {type(response)}")
            if type(response) is list:
                print("\n\n" + json.dumps(response, indent=4))

            elif type(response) is str:
                print("\n\n" + response)
            assert True

        except (
                    PyegeriaException
        ) as e:
            print_exception_table(e)
            assert False, "Invalid request"

        finally:
            g_client.close_session()




    # def test_get_terms_for_glossary(self, server: str = good_view_server_1):
    #     server_name = server
    #     try:
    #         g_client = GlossaryManager(
    #             server_name, self.good_platform1_url, user_id="erinoverview"
    #         )
    #
    #         token = g_client.create_egeria_bearer_token(self.good_user_2, "secret")
    #         # glossary_guid = "f9b78b26-6025-43fa-9299-a905cc6d1575"  # This is the sustainability glossary
    #         # glossary_guid = "706ba88d-d0bb-42da-82d9-385b13516b34" # Teddy Bear Drop Foot
    #         # glossary_guid = (
    #         #     "c13e22d5-756a-4b54-b784-14037ee3dfc4"  # larger sustainability glossary
    #         # )
    #         glossary_guid = "65b43594-b105-49da-a59e-269e1fdb2f76"
    #
    #         start_time = time.perf_counter()
    #         response = g_client.get_terms_for_glossary(
    #             glossary_guid
    #         )
    #         print(f"Duration is {time.perf_counter()-start_time} seconds")
    #         print(f"type is {type(response)}")
    #         if type(response) is list:
    #             print("\n\n" + json.dumps(response, indent=4))
    #             count = len(response)
    #             print(f"Found {count} terms")
    #         elif type(response) is str:
    #             print("\n\n" + response)
    #         assert True
    #
    #     except (
    #                 PyegeriaException
    #     ) as e:
    #         print_exception_table(e)
    #         assert False, "Invalid request"
    #
    #     finally:
    #         g_client.close_session()

    def test_export_glossary_to_csv(self):
        try:
            g_client = GlossaryManager(
                self.good_view_server_1,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd=self.good_user_2_pwd,
            )

            token = g_client.create_egeria_bearer_token(
                self.good_user_2, self.good_user_2_pwd
            )
            start_time = time.perf_counter()
            glossary_guid = "70ae4d54-05bb-4411-96e6-697d0640a10e"
            file_name = f"/Users/dwolfson/localGit/test_export-{glossary_guid}.csv"
            response = g_client.export_glossary_to_csv(glossary_guid, file_name)
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            print(f"\n\nExported {response} rows to {file_name}")
            assert True
        except (
                    PyegeriaException
        ) as e:
            print_exception_table(e)
            assert False, "Invalid request"
        finally:
            g_client.close_session()

    def test_get_glossary_for_term(self, server: str = good_view_server_1):
        server_name = server

        try:
            g_client = GlossaryManager(
                server_name, self.good_platform1_url, user_id="erinoverview"
            )

            token = g_client.create_egeria_bearer_token(self.good_user_2, "secret")
            term_guid = "5467f5d4-d821-4fc2-ad71-8af7181826ff"
            response = g_client.get_glossary_for_term(term_guid)

            print(f"type is {type(response)}")
            if type(response) is dict:
                print("\n\n" + json.dumps(response, indent=4))
            elif type(response) is str:
                print("\n\n" + response)
            assert True

        except (
                    PyegeriaException
        ) as e:
            print_exception_table(e)
            assert False, "Invalid request"

        finally:
            g_client.close_session()

    def test_get_terms_by_name(self, server: str = good_view_server_2):
        server_name = server

        try:
            g_client = GlossaryManager(
                server_name, self.good_platform1_url, user_id="erinoverview"
            )

            token = g_client.create_egeria_bearer_token(self.good_user_2, "secret")
            term_name = "Calico"
            glossary_guid = "70ae4d54-05bb-4411-96e6-697d0640a10e"
            response = g_client.get_terms_by_name(term_name)

            print(f"type is {type(response)}")
            if type(response) is list:
                print("\n\n" + json.dumps(response, indent=4))
            elif type(response) is str:
                print("\n\n" + response)
            assert True

        except (
                    PyegeriaException
        ) as e:
            print_exception_table(e)
            assert False, "Invalid request"

        finally:
            g_client.close_session()

    def test_get_terms_by_guid(self, server: str = good_view_server_2):
        server_name = server

        try:
            g_client = GlossaryManager(
                server_name, self.good_platform1_url, user_id="erinoverview"
            )

            token = g_client.create_egeria_bearer_token(self.good_user_2, "secret")
            term_guid = "7bb48da4-f242-4deb-9293-375bb67bbbcf"

            response = g_client.get_term_by_guid(term_guid, output_format = 'JSON')

            print(f"type is {type(response)}")
            if type(response) is dict:
                print("\n\n" + json.dumps(response, indent=4))
                print(
                    f"Term name is: {response['properties']['displayName']}"
                )
            elif type(response) is str:
                print("\n\n" + response)
            assert True

        except (
                    PyegeriaException
        ) as e:
            print_exception_table(e)
            assert False, "Invalid request"

        finally:
            g_client.close_session()

    def test_find_glossary_terms(self):
        try:
            g_client = GlossaryManager(
                self.good_view_server_2, self.good_platform1_url, self.good_user_2
            )

            token = g_client.create_egeria_bearer_token(self.good_user_2, "secret")
            # glossary_guid = "017dee20-b8ce-4d74-854b-f2a888a082cd" # small-email glossary
            # glossary_guid = (
            #     "f9b78b26-6025-43fa-9299-a905cc6d1575"  # sustainability glossary
            # )
            glossary_guid = None
            start_time = time.perf_counter()
            response = g_client.find_glossary_terms(
                "*",
                True,
                False,
                True,
                output_format="JSON",
                report_spec = "Basic-Terms"
            )
            print(f"Duration is {time.perf_counter() - start_time} seconds")
            print(f"Number of terms is: {len(response)}")
            if type(response) is list:
                print("\n\n" + json.dumps(response, indent=4))
                # print_json_list_as_table(response)
                count = len(response)
                print(f"Found {count} terms")
            elif type(response) is str:
                print("\n\n" + response)
            assert True

        except (
                    PyegeriaException
        ) as e:
            print_exception_table(e)
            assert False, "Invalid request"

        finally:
            g_client.close_session()

    def test_update_term(self):
        try:
            g_client = GlossaryManager(
                self.good_view_server_2, self.good_platform1_url, self.good_user_2
            )

            token = g_client.create_egeria_bearer_token(self.good_user_2, "secret")
            term_guid = "ecb57c19-fb1c-42f4-ab1a-c85b6fe753ea"  # meow
            body = {
                "class": "ReferenceableUpdateRequestBody",
                "elementProperties": {
                    "class": "GlossaryTermProperties",
                    "description": "With alieas",
                    "aliases" : ["t1","xt1"]
                },
            }

            start_time = time.perf_counter()
            g_client.update_term(term_guid, body, True)
            print(f"Duration is {time.perf_counter() - start_time} seconds")

            assert True

        except (
                    PyegeriaException
        ) as e:
            print_exception_table(e)
            assert False, "Invalid request"

        finally:
            g_client.close_session()


    def test_load_terms_from_csv(self):
        try:
            g_client = GlossaryManager(
                self.good_view_server_2, self.good_platform1_url, self.good_user_3
            )
            token = g_client.create_egeria_bearer_token(self.good_user_3, "secret")
            glossary = "Egeria-Markdown"
            file_path = "/Users/dwolfson/localGit/egeria-v5-1/egeria-workspaces/exchange/loading-bay/glossary"
            file_name = "Test1.om-terms"
            response = g_client.load_terms_from_csv_file(glossary, file_name, file_path)
            print(f"type is {type(response)}")
            if type(response) is list:
                print("\n\n" + json.dumps(response, indent=4))
            elif type(response) is str:
                print("\n\n" + response)
            assert True

        except (
             PyegeriaException,
            FileNotFoundError,
        ) as e:
            print_exception_table(e)
            assert False, "Invalid request"
        except Exception as e:
            print(e)

        finally:
            g_client.close_session()

    def test_delete_term(self):
        try:
            g_client = GlossaryManager(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd=self.good_user_2_pwd,
            )

            token = g_client.create_egeria_bearer_token(
                self.good_user_2, self.good_user_2_pwd
            )
            start_time = time.perf_counter()
            term_guid = "675d210c-3801-4555-9abe-99bdb45f000a"
            g_client.delete_term(term_guid)
            duration = time.perf_counter() - start_time

            print(f"\n\tDuration was {duration} seconds")
            print(f"\n\nDeleted term {term_guid}")
            assert True
        except (
                    PyegeriaException
        ) as e:
            print_exception_table(e)
            assert False, "Invalid request"
        finally:
            g_client.close_session()


    def test_add_term_to_folder(self, server: str = good_view_server_2):
        try:
            g_client = GlossaryManager(
                server, self.good_platform1_url, user_id=self.good_user_2
                )

            token = g_client.create_egeria_bearer_token(self.good_user_2, "secret")


            term = "d9476777-29de-4c3e-b999-5a0cac40b438"
            folder = "42dbd940-5a33-4620-b7af-af21404921fb"
            relationship_type = "Antonym"
            body = {
                "class": "RelationshipRequestBody",
                "properties": {
                    "class": "GlossaryTermRelationship",
                    # "confidence": 10,
                    # "description": "Why not",
                    # "status": "DRAFT",
                    # "steward": "Martha"
                }
            }

            g_client.add_term_to_folder(folder, term)

            assert True

        except (
                InvalidParameterException,
                PropertyServerException,
                UserNotAuthorizedException,
                ) as e:
            print_exception_table(e)
            assert False, "Invalid request"

        finally:
            g_client.close_session()

    def test_add_relationship_between_terms(self, server: str = good_view_server_2):
        try:
            g_client = GlossaryManager(
                server, self.good_platform1_url, user_id=self.good_user_2
                )

            token = g_client.create_egeria_bearer_token(self.good_user_2, "secret")


            guid1 = "16fe1b6f-b66c-490e-bc23-a47d0510b433"
            guid2 = "ecf8b8d6-e593-4240-839c-019b820f1897"
            relationship_type = "Antonym"
            body = {
                "class": "NewRelationshipRequestBody",
                "properties": {
                    "class": "GlossaryTermRelationship"
                    # "confidence": 10,
                    # "description": "Why not",
                    # "status": "DRAFT",
                    # "steward": "Martha"
                }
            }

            g_client.add_relationship_between_terms(guid1, guid2, relationship_type)

            assert True

        except (PyegeriaException
                ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"
        except ValidationError as e:
            print_validation_error(e)

        finally:
            g_client.close_session()

    def test_update_relationship_between_terms(self, server: str = good_view_server_2):
        try:
            g_client = GlossaryManager(
                server, self.good_platform1_url, user_id=self.good_user_2
                )

            token = g_client.create_egeria_bearer_token(self.good_user_2, "secret")
            guid1 = "2852b4e1-4445-44ee-b3aa-dbd1e577cdcb"
            guid2 = "4961c79b-b040-4597-b0d3-5d32dd5f8935"
            relationship_type = "Synonym"
            body = {
                "class": "RelationshipRequestBody",
                "properties": {
                    "class": "GlossaryTermRelationship",
                    "confidence": 15,
                    "description": "Why not",
                    "status": "DRAFT",
                    "steward": "Martha"
                }
            }


            g_client.update_relationship_between_terms(guid1, guid2, relationship_type, body)

            assert True

        except (
                InvalidParameterException,
                PropertyServerException,
                UserNotAuthorizedException,
                ) as e:
            print_exception_table(e)
            assert False, "Invalid request"

        finally:
            g_client.close_session()

    def test_remove_relationship_between_terms(self, server: str = good_view_server_2):
        try:
            g_client = GlossaryManager(
                server, self.good_platform1_url, user_id=self.good_user_2
                )

            token = g_client.create_egeria_bearer_token(self.good_user_2, "secret")
            # guid1 = "2852b4e1-4445-44ee-b3aa-dbd1e577cdcb"
            # guid2 = "4961c79b-b040-4597-b0d3-5d32dd5f8935"
            guid1 = "46d4ad19-7627-4a8a-91b2-5b0a65ae328c"
            guid2 = "49e07289-d56f-4c45-9b87-e3d86c11d056"
            relationship_type = "Synonym"



            g_client.remove_relationship_between_terms(guid1, guid2, relationship_type)

            assert True

        except (
                InvalidParameterException,
                PropertyServerException,
                UserNotAuthorizedException,
                ) as e:
            print_exception_table(e)
            assert False, "Invalid request"

        finally:
            g_client.close_session()

    def test_remove_relationship_between_terms(self, server: str = good_view_server_2):
        try:
            g_client = GlossaryManager(
                server, self.good_platform1_url, user_id=self.good_user_2
                )

            token = g_client.create_egeria_bearer_token(self.good_user_2, "secret")
            # guid1 = "2852b4e1-4445-44ee-b3aa-dbd1e577cdcb"
            # guid2 = "4961c79b-b040-4597-b0d3-5d32dd5f8935"
            guid1 = "46d4ad19-7627-4a8a-91b2-5b0a65ae328c"
            guid2 = "49e07289-d56f-4c45-9b87-e3d86c11d056"
            relationship_type = "Synonym"

            resp = g_client.make_request("POST","http://localhost:8085/egeria/get_origin")
            print (resp)
            assert True

        except (
                InvalidParameterException,
                PropertyServerException,
                UserNotAuthorizedException,
                ) as e:
            print_exception_table(e)
            assert False, "Invalid request"

        finally:
            g_client.close_session()


"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

Template manager is a view service that supports managing metadata elements using templates.

"""

import asyncio

from pyegeria._client import Client
from pyegeria._globals import default_time_out, NO_ELEMENTS_FOUND
from pyegeria.utils import body_slimmer


class TemplateManager(Client):
    """Client to issue Template Manager requests.

    Attributes:

        view_server : str
                Name of the server to use.
        platform_url : str
            URL of the server platform to connect to
        user_id : str
            The identity of the user calling the method - this sets a default optionally used by the methods
            when the user doesn't pass the user_id on a method call.
        user_pwd: str
            The password associated with the user_id. Defaults to None
        token: str, optional
            Bearer token

    Methods:

    """

    def __init__(
        self,
        view_server: str,
        platform_url: str,
        user_id: str,
        user_pwd: str = None,
        token: str = None,
        time_out: int = default_time_out,
    ):
        self.view_server = view_server
        self.time_out = time_out
        Client.__init__(self, view_server, platform_url, user_id, user_pwd, token=token)
        self.command_root = f"{self.platform_url}/servers/{self.view_server}/api/open-metadata/template-manager"

    #
    #   Maintain the metadata elements that makes up the template
    #
    async def _async_create_metadata_element_in_store(self, body: dict) -> str:
        """Create a new metadata element in the metadata store.  The type name comes from the open metadata types.
        The selected type also controls the names and types of the properties that are allowed.
        This version of the method allows access to advanced features such as multiple states and
        effectivity dates. Async version.

        Parameters
        ----------
        body : dict
            The definition of the element to create. A sample is below.

        Returns
        -------
           str: If successful, the GUID of the element created; otherwise None is returned.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        =====

        Example of the body:

        {
          "class" : "NewOpenMetadataElementRequestBody",
          "externalSourceGUID" : "666",
          "externalSourceName" : "radio1",
          "typeName" : "",
          "initialStatus" : "ACTIVE",
          "initialClassifications" : {},
          "anchorGUID" : "",
          "isOwnAnchor" : false,
          "effectiveFrom" : "{{$isoTimestamp}}",
          "effectiveTo": "{{$isoTimestamp}}",
          "properties" : {},
          "parentGUID" : "",
          "parentRelationshipTypeName" : "",
          "parentRelationshipProperties" : {},
          "parentAtEnd1" : true,
          "effectiveTime" : "{{$isoTimestamp}}"
        }

        """

        url = f"{self.command_root}/metadata-elements"
        response = await self._async_make_request("POST", url, body_slimmer(body))
        guid = response.json().get("guid", None)
        return guid

    def create_metadata_element_in_store(self, body: dict) -> str:
        """Create a new metadata element in the metadata store.  The type name comes from the open metadata types.
        The selected type also controls the names and types of the properties that are allowed.
        This version of the method allows access to advanced features such as multiple states and
        effectivity dates.

        Parameters
        ----------
        body : dict
            The definition of the element to create. A sample is below.

        Returns
        -------
           str: If successful, the GUID of the element created; otherwise None is returned.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        =====

        Example of the body:

        {
          "class" : "NewOpenMetadataElementRequestBody",
          "externalSourceGUID" : "666",
          "externalSourceName" : "radio1",
          "typeName" : "",
          "initialStatus" : "ACTIVE",
          "initialClassifications" : {},
          "anchorGUID" : "",
          "isOwnAnchor" : false,
          "effectiveFrom" : "{{$isoTimestamp}}",
          "effectiveTo": "{{$isoTimestamp}}",
          "properties" : {},
          "parentGUID" : "",
          "parentRelationshipTypeName" : "",
          "parentRelationshipProperties" : {},
          "parentAtEnd1" : true,
          "effectiveTime" : "{{$isoTimestamp}}"
        }

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(
            self._async_create_metadata_element_in_store(body)
        )
        return response

    async def _async_create_metadata_element_from_template(self, body: dict) -> str:
        """Create a new metadata element in the metadata store using a template.  The type name comes from the
        open metadata types. The selected type also controls the names and types of the properties that are allowed.
        Async version.

        Parameters
        ----------
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "TemplateRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "typeName" : "",
          "templateGUID" : "",
          "anchorGUID" : "",
          "isOwnAnchor" : false,
          "effectiveFrom" : "{{$isoTimestamp}}",
          "effectiveTo": "{{$isoTimestamp}}",
          "replacementProperties" : { },
          "placeholderProperties" : {
             "placeholderName1" : "placeholderValue1",
             "placeholderName2" : "placeholderValue2"
          },
          "parentGUID" : "",
          "parentRelationshipTypeName" : "",
          "parentRelationshipProperties" : {},
          "parentAtEnd1" : true,
          "effectiveTime" : "{{$isoTimestamp}}"
        }

        """
        url = f"{self.command_root}/metadata-elements/from-templates"

        response = await self._async_make_request("POST", url, body_slimmer(body))
        guid = response.json().get("guid", None)
        return guid

    def create_metadata_element_from_template(self, body: dict) -> str:
        """Create a new metadata element in the metadata store using a template.  The type name comes from the
        open metadata types. The selected type also controls the names and types of the properties that are allowed.

        Parameters
        ----------
        body : dict
            dict containing the definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "UpdatePropertiesRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
        }

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(
            self._async_create_metadata_element_from_template(body)
        )
        return response

    async def _async_update_metadata_element_in_store(
        self, element_guid: str, body: dict
    ) -> None:
        """Update the properties of a specific metadata element.  The properties must match the type definition
        associated with the metadata element when it was created.  However, it is possible to update a few
        properties, or replace all them by the value used in the replaceProperties flag.
        Async version.

        Parameters
        ----------
        element_guid : str
            The identity of the metadata element to update.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "TemplateRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "typeName" : "",
          "templateGUID" : "",
          "anchorGUID" : "",
          "isOwnAnchor" : false,
          "effectiveFrom" : "{{$isoTimestamp}}",
          "effectiveTo": "{{$isoTimestamp}}",
          "replacementProperties" : { },
          "placeholderProperties" : {
             "placeholderName1" : "placeholderValue1",
             "placeholderName2" : "placeholderValue2"
          },
          "parentGUID" : "",
          "parentRelationshipTypeName" : "",
          "parentRelationshipProperties" : {},
          "parentAtEnd1" : true,
          "effectiveTime" : "{{$isoTimestamp}}"
        }

        """
        url = f"{self.command_root}/metadata-elements/{element_guid}/update-properties"
        await self._async_make_request("POST", url, body_slimmer(body))
        return

    def update_metadata_element_in_store(self, element_guid: str, body: dict) -> None:
        """Update the properties of a specific metadata element.  The properties must match the type definition
        associated with the metadata element when it was created.  However, it is possible to update a few
        properties, or replace all them by the value used in the replaceProperties flag.

        Parameters
        ----------
        element_guid : str
            The identity of the metadata element to update.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "TemplateRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "typeName" : "",
          "templateGUID" : "",
          "anchorGUID" : "",
          "isOwnAnchor" : false,
          "effectiveFrom" : "{{$isoTimestamp}}",
          "effectiveTo": "{{$isoTimestamp}}",
          "replacementProperties" : { },
          "placeholderProperties" : {
             "placeholderName1" : "placeholderValue1",
             "placeholderName2" : "placeholderValue2"
          },
          "parentGUID" : "",
          "parentRelationshipTypeName" : "",
          "parentRelationshipProperties" : {},
          "parentAtEnd1" : true,
          "effectiveTime" : "{{$isoTimestamp}}"
        }

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_update_metadata_element_in_store(element_guid, body)
        )
        return

    async def _async_update_metadata_element_status_in_store(
        self, element_guid: str, body: dict
    ) -> None:
        """Update the status of a specific metadata element. The new status must match a status value that is defined
        for the element's type assigned when it was created. Async version.

        Parameters
        ----------
        element_guid : str
            The identity of the metadata element to update.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "UpdateStatusRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
        }

        """
        url = f"{self.command_root}/metadata-elements/{element_guid}/update-status"
        await self._async_make_request("POST", url, body_slimmer(body))
        return

    def update_metadata_element_status_in_store(
        self, element_guid: str, body: dict
    ) -> None:
        """Update the status of a specific metadata element. The new status must match a status value that is defined
        for the element's type assigned when it was created.

        Parameters
        ----------
        element_guid : str
            The identity of the metadata element to update.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "UpdateStatusRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
        }

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_update_metadata_element_status_in_store(element_guid, body)
        )
        return

    async def _async_update_metadata_element_effectivity_in_store(
        self, element_guid: str, body: dict
    ) -> None:
        """Update the effectivity dates control the visibility of the element through specific APIs.
            Async version.

        Parameters
        ----------
        element_guid : str
            The identity of the metadata element to update.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "UpdateEffectivityDatesRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "effectiveFrom" : "{{$isoTimestamp}}",
          "effectiveTo": "{{$isoTimestamp}}",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
        }

        """
        url = f"{self.command_root}/metadata-elements/{element_guid}/update-effectivity"
        await self._async_make_request("POST", url, body_slimmer(body))
        return

    def update_metadata_element_effectivity_in_store(
        self, element_guid: str, body: dict
    ) -> None:
        """Update the effectivity dates control the visibility of the element through specific APIs.
            Async version.

        Parameters
        ----------
        element_guid : str
            The identity of the metadata element to update.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "UpdateEffectivityDatesRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "effectiveFrom" : "{{$isoTimestamp}}",
          "effectiveTo": "{{$isoTimestamp}}",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
        }

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_update_metadata_element_effectivity_in_store(element_guid, body)
        )
        return

    async def _async_delete_metadata_element_in_store(
        self, element_guid: str, body: dict, cascade:bool = False
    ) -> None:
        """Delete a metadata element.
            Async version.

        Parameters
        ----------
        element_guid : str
            The identity of the metadata element to update.
        body : dict
            The definition of the element to create. A sample is the notes below.
        cascade : bool, optional, defaults to False
            If true, delete all anchored elements as well.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "UpdateRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
        }

        """
        cascade_str = str(cascade).lower()
        url = f"{self.command_root}/metadata-elements/{element_guid}/delete?cascadedDelete={cascade_str}"
        await self._async_make_request("POST", url, body_slimmer(body))
        return

    def delete_metadata_element_in_store(self, element_guid: str, body: dict, cascade:bool = False) -> None:
        """Delete a metadata element.

        Parameters
        ----------
        element_guid : str
            The identity of the metadata element to update.
        body : dict
            The definition of the element to create. A sample is the notes below.
        cascade : bool, optional, defaults to False
            If true, delete all anchored elements as well.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "UpdateRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
        }

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_delete_metadata_element_in_store(element_guid, body, cascade)
        )
        return

    async def _async_archive_metadata_element_in_store(
        self, element_guid: str, body: dict
    ) -> None:
        """Archive a specific metadata element.
            Async version.

        Parameters
        ----------
        element_guid : str
            The identity of the metadata element to update.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "ArchiveRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "archiveProperties" : {
            "archiveDate" : "{{$isoTimestamp}}",
            "archiveProcess" : "",
            "archiveProperties": {
               "propertyName1" : "propertyValue1",
               "propertyName2" : "propertyValue2"
            }
          },
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
        }

        """
        url = f"{self.command_root}/metadata-elements/{element_guid}/archive"
        await self._async_make_request("POST", url, body_slimmer(body))
        return

    def archive_metadata_element_in_store(self, element_guid: str, body: dict) -> None:
        """Archive a specific metadata element.

        Parameters
        ----------
        element_guid : str
            The identity of the metadata element to update.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

          "class" : "ArchiveRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "archiveProperties" : {
            "archiveDate" : "{{$isoTimestamp}}",
            "archiveProcess" : "",
            "archiveProperties": {
               "propertyName1" : "propertyValue1",
               "propertyName2" : "propertyValue2"
            }
          },
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
        }

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_archive_metadata_element_in_store(element_guid, body)
        )
        return

    async def _async_classify_metadata_element_in_store(
        self, element_guid: str, classification: str, body: dict
    ) -> None:
        """Add a new classification to the metadata element.  Note that only one classification with the same name can
            be attached to a metadata element. Async version.

        Parameters
        ----------
        element_guid : str
            The identity of the metadata element to update.
        classification : str
            The classification name to apply.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "NewClassificationRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
        }

        """
        url = f"{self.command_root}/metadata-elements/{element_guid}/classifications/{classification}"
        await self._async_make_request("POST", url, body_slimmer(body))
        return

    def classify_metadata_element_in_store(
        self, element_guid: str, classification: str, body: dict
    ) -> None:
        """Add a new classification to the metadata element.  Note that only one classification with the same name can
            be attached to a metadata element.

        Parameters
        ----------
        element_guid : str
            The identity of the metadata element to update.
        classification : str
            The classification name to apply.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "NewClassificationRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
        }

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_classify_metadata_element_in_store(
                element_guid, classification, body
            )
        )
        return

    async def _async_reclassify_metadata_element_in_store(
        self, element_guid: str, classification: str, body: dict
    ) -> None:
        """Update the properties of a classification that is currently attached to a specific metadata element.
            Async version.

        Parameters
        ----------
        element_guid : str
            The identity of the metadata element to update.
        classification: str
            The classification name to apply.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "UpdatePropertiesRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
        }

        """
        url = f"{self.command_root}/metadata-elements/{element_guid}/classifications/{classification}/update-properties"
        await self._async_make_request("POST", url, body_slimmer(body))
        return

    def reclassify_metadata_element_in_store(
        self, element_guid: str, classification: str, body: dict
    ) -> None:
        """Update the properties of a classification that is currently attached to a specific metadata element.

        Parameters
        ----------
        element_guid : str
            The identity of the metadata element to update.
        classification: str
            The classification name to apply.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "UpdatePropertiesRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
        }

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_reclassify_metadata_element_in_store(
                element_guid, classification, body
            )
        )
        return

    async def _async_update_classification_effectivity_in_store(
        self, element_guid: str, classification: str, body: dict
    ) -> None:
        """Update the effectivity dates of a specific classification attached to a metadata element.
            The effectivity dates control the visibility of the classification through specific APIs.
            Async version.

        Parameters
        ----------
        element_guid : str
            The identity of the metadata element to update.
        classification: str
            The classification name to apply.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "UpdateEffectivityDatesRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "effectiveFrom" : "{{$isoTimestamp}}",
          "effectiveTo": "{{$isoTimestamp}}",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
        }
        """
        url = f"{self.command_root}/metadata-elements/{element_guid}/classifications/{classification}/update-effectivity"
        await self._async_make_request("POST", url, body_slimmer(body))
        return

    def update_classification_effectivity_in_store(
        self, element_guid: str, classification: str, body: dict
    ) -> None:
        """Update the effectivity dates of a specific classification attached to a metadata element.
            The effectivity dates control the visibility of the classification through specific APIs.


        Parameters
        ----------
        element_guid : str
            The identity of the metadata element to update.
        classification: str
            The classification name to apply.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "UpdateEffectivityDatesRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "effectiveFrom" : "{{$isoTimestamp}}",
          "effectiveTo": "{{$isoTimestamp}}",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
        }
        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_update_classification_effectivity_in_store(
                element_guid, classification, body
            )
        )
        return

    async def _async_declassify_metadata_element_in_store(
        self, element_guid: str, classification: str, body: dict
    ) -> None:
        """Remove the named classification from a specific metadata element. Async version.

        Parameters
        ----------
        element_guid : str
            The identity of the metadata element to update.
        classification: str
            The classification name to apply.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "UpdateRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
        }
        """
        url = f"{self.command_root}/metadata-elements/{element_guid}/classifications/{classification}/delete"
        await self._async_make_request("POST", url, body_slimmer(body))
        return

    def declassify_metadata_element_in_store(
        self, element_guid: str, classification: str, body: dict
    ) -> None:
        """Remove the named classification from a specific metadata element.

        Parameters
        ----------
        element_guid : str
            The identity of the metadata element to update.
        classification: str
            The classification name to apply.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "UpdateRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
        }
        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_declassify_metadata_element_in_store(
                element_guid, classification, body
            )
        )
        return

    async def _async_create_related_elements_in_store(self, body: dict) -> str:
        """Create a relationship between two metadata elements.  It is important to put the right element at each end
            of the relationship according to the type definition since this will affect how the relationship is
            interpreted. Async version.

        Parameters
        ----------
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           str containing the relationship GUID.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "NewRelatedElementsRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
          "typeName": "string",
          "metadataElement1GUID": "string",
          "metadataElement2GUID": "string",
        }
        """
        url = f"{self.command_root}/related-elements"
        response = await self._async_make_request("POST", url, body_slimmer(body))
        guid = response.json().get("guid", None)
        return guid

    def create_related_elements_in_store(self, body: dict) -> str:
        """Create a relationship between two metadata elements.  It is important to put the right element at each end
            of the relationship according to the type definition since this will affect how the relationship is
            interpreted.

        Parameters
        ----------
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           str containing the relationship GUID.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "NewRelatedElementsRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
          "typeName": "string",
          "metadataElement1GUID": "string",
          "metadataElement2GUID": "string",
        }
        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(
            self._async_create_related_elements_in_store(body)
        )
        return response

    async def _async_update_related_elements_in_store(
        self, relationship_guid: str, body: dict
    ) -> None:
        """Update the properties associated with a relationship. Async version.

        Parameters
        ----------
        relationship_guid : str
            The identity of the relationship to update.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "UpdatePropertiesRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "typeName": "string",
          "metadataElement1GUID": "string",
          "metadataElement2GUID": "string",
          "effectiveTime" : "{{$isoTimestamp}}"
        }
        """
        url = f"{self.command_root}/related-elements/{relationship_guid}/update-properties"
        await self._async_make_request("POST", url, body_slimmer(body))
        return

    def update_related_elements_in_store(
        self, relationship_guid: str, body: dict
    ) -> None:
        """Update the properties associated with a relationship. Async version.

        Parameters
        ----------
        relationship_guid : str
            The identity of the relationship to update.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "UpdatePropertiesRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "typeName": "string",
          "metadataElement1GUID": "string",
          "metadataElement2GUID": "string",
          "effectiveTime" : "{{$isoTimestamp}}"
        }
        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_update_related_elements_in_store(relationship_guid, body)
        )
        return

    async def _async_update_related_elements_effectivity_in_store(
        self, relationship_guid: str, body: dict
    ) -> None:
        """Update the effectivity dates of a specific relationship between metadata elements.
            The effectivity dates control the visibility of the classification through specific APIs.
            Async version.

        Parameters
        ----------
        relationship_guid : str
            The identity of the relationship to update.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "UpdateEffectivityDatesRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "effectiveFrom" : "{{$isoTimestamp}}",
          "effectiveTo": "{{$isoTimestamp}}",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
        }
        """
        url = f"{self.command_root}/metadata-elements/related-elements/{relationship_guid}/update-effectivity"
        await self._async_make_request("POST", url, body_slimmer(body))
        return

    def update_related_elements_effectivity_in_store(
        self, relationship_guid: str, body: dict
    ) -> None:
        """Update the effectivity dates of a specific relationship between metadata elements.
            The effectivity dates control the visibility of the classification through specific APIs.

        Parameters
        ----------
        relationship_guid : str
            The identity of the relationship to update.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "UpdateEffectivityDatesRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "effectiveFrom" : "{{$isoTimestamp}}",
          "effectiveTo": "{{$isoTimestamp}}",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
        }
        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_update_related_elements_effectivity_in_store(
                relationship_guid, body
            )
        )
        return

    async def _async_delete_related_elements_in_store(
        self, relationship_guid: str, body: dict
    ) -> None:
        """Delete a relationship between two metadata elements. Async version.

        Parameters
        ----------
        relationship_guid : str
            The identity of the relationship to delete.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "UpdateRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
        }
        """
        url = f"{self.command_root}/metadata-elements/related-elements/{relationship_guid}/delete"
        await self._async_make_request("POST", url, body_slimmer(body))
        return

    def delete_related_elements_in_store(
        self, relationship_guid: str, body: dict
    ) -> None:
        """Delete a relationship between two metadata elements.

        Parameters
        ----------
        relationship_guid : str
            The identity of the relationship to delete.
        body : dict
            The definition of the element to create. A sample is the notes below.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        Notes
        -----
        Example of the body:

        {
          "class" : "UpdateRequestBody",
          "externalSourceGUID" :  "",
          "externalSourceName" : "",
          "forLineage" : false,
          "forDuplicateProcessing" : false,
          "effectiveTime" : "{{$isoTimestamp}}"
        }
        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_delete_related_elements_in_store(relationship_guid, body)
        )
        return


if __name__ == "__main__":
    print("Main-Template Manager")



"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.



This module is for testing the core OMAG config class and methods
The routines assume that pytest is being used as the test tool and framework.

A running Egeria environment is needed to run these tests.

"""

import asyncio
import json
import os
import time
from contextlib import nullcontext as does_not_raise

import pytest
from pydantic import ValidationError

from pyegeria import Client2, PyegeriaException, print_basic_exception, print_validation_error
from pyegeria._exceptions import (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
)
from pyegeria.core_omag_server_config import CoreServerConfig
# from pyegeria.feedback_manager_omvs import FeedbackManager

disable_ssl_warnings = True


TESTING_EGERIA_PLATFORM_URL = os.environ.get(
    "TESTING_EGERIA_PLATFORM_URL", "https://localhost:9443"
)

view_server = "qs-view-server"
user = "erinoverview"
password = "secret"

fm_client = Client2(view_server, TESTING_EGERIA_PLATFORM_URL, user)


bearer_token = fm_client.create_egeria_bearer_token(user, password)

element_guid = "fb6dd0e5-afe1-4144-9b7d-e4559e7b09d2"
term_guid = element_guid

tag_for_testing = {
    "isPrivateTag": False,
    "name": "testing tag",
    "description": "This is a tag for testing purpose",
}

rating = {
    "class": "RatingProperties",
    "starRating": "TWO_STARS",
    "review": "This is my review",
}

notelog_for_testing = {
    "class": "NoteLogProperties",
    "qualifiedName": "test-note-log",
    "name": "Test Note Log",
    "description": "This is a note log for testing purposes",
    "additionalProperties": {"skyColor": "blue", "bestNoteLogEver": "this one"},
}

updated_notelog_for_testing = {
    "class": "ReferenceableUpdateRequestBody",
    "properties": {
        "class": "NoteLogProperties",
        "qualifiedName": "test-note-log",
        "name": "Test Note Log",
        "description": "This is a note log for testing purposes, with updates",
        "additionalProperties": {
            "waterColor": "blue",
            "bestNoteLogEver": "still this one",
        },
    },
}

note_for_testing = {
    "class": "NoteProperties",
    "qualifiedName": "ready for testing",
    "title": "Ready for Testing",
    "text": "This element has been reviewed and is ready for further testing",
    "additionalProperties": {"status": "Testing Ready", "isCritical?": "No"},
}

updated_note_for_testing = {
    "class": "ReferenceableUpdateRequestBody",
    "properties": {
        "class": "NoteProperties",
        "qualifiedName": "ready for testing",
        "title": "Ready for Testing",
        "text": "This element has been reviewed and is ready for further testing after subsequent updates",
        "additionalProperties": {"hasBeenUpdated?": "not sure", "isCritical?": "Yes"},
    },
}


standard_comment = {
    "class": "NewAttachmentRequestBody",
    "properties": {
        "class": "CommentProperties",
        "qualifiedName": "ExampleStandardComment",
        "description": "This is just an example STANDARD comment",
        "comment-type": "STANDARD_COMMENT",
        "additionalProperties": {
            "propertyName1": "property value 1",
            "propertyNameN": "property value N",
        },
    },
}

updated_standard_comment = {
    "class": "ReferenceableUpdateRequestBody",
    "properties": {
        "class": "CommentProperties",
        "qualifiedName": "ExampleStandardComment",
        "description": "This is just an example of an updatedSTANDARD comment",
        "comment-type": "STANDARD_COMMENT",
        "additionalProperties": {
            "propertyName2": "property value 2",
            "propertyNameN": "property value N",
        },
    },
}

reply_comment = {
    "class": "ReferenceableUpdateRequestBody",
    "properties": {
        "class": "CommentProperties",
        "qualifiedName": "ExampleReplyComment",
        "description": "This is just an example STANDARD comment reply",
        "comment-type": "STANDARD_COMMENT",
        "additionalProperties": {
            "propertyName1": "property value 1",
            "propertyNameN": "property value N",
        },
    },
}


question_comment = {
    "class": "ReferenceableUpdateRequestBody",
    "properties": {
        "class": "CommentProperties",
        "qualifiedName": "ExampleQuestionComment",
        "description": "This is just an example Question comment. We would pose some question here.",
        "commentType": "QUESTION",
        "additionalProperties": {
            "propertyName1": "property value 1",
            "propertyNameN": "property value N",
        },
    },
}

answer_comment = {
    "class": "ReferenceableUpdateRequestBody",
    "properties": {
        "class": "CommentProperties",
        "qualifiedName": "ExampleAnswerComment",
        "description": "This is just an example Answer comment. We would propose an answer to a question question here.",
        "commentType": "ANSWER",
        "additionalProperties": {
            "propertyName1": "property value 1",
            "propertyNameN": "property value N",
        },
    },
}


def jprint(info, comment=None):
    if comment:
        print(comment)
    print(json.dumps(info, indent=2))


def valid_guid(guid):
    if (guid is None) or (type(guid) is not str):
        return False
    else:
        return True


def test_add_comment_reply_with_private_reply():
    try:
        comment_response = fm_client.add_comment_to_element(element_guid, body=standard_comment)
        reply_response = fm_client.add_comment_reply(
            term_guid, comment_response, body=reply_comment
        )
        fm_client.remove_comment(comment_response,reply_response)
        fm_client.remove_comment(element_guid, comment_response)
        assert reply_response["relatedHTTPCode"] == 200
        assert reply_response["class"] == "GUIDResponse"
        assert "guid" in reply_response
    except PyegeriaException as e:
        print_basic_exception(e)
#
## test_add_comment_reply test
#
def test_add_comment_reply():
    comment_response = fm_client.add_comment_to_element(term_guid, body=standard_comment)
    reply_response = fm_client.add_comment_reply(
        term_guid, comment_response["guid"], body=reply_comment
    )
    fm_client.remove_comment(element_guid,reply_response["guid"])
    fm_client.remove_comment(comment_response["guid"])
    assert reply_response["relatedHTTPCode"] == 200
    assert reply_response["class"] == "GUIDResponse"
    assert "guid" in reply_response





def test_add_comment_reply_with_access_service_specified():
    comment_response = fm_client.add_comment_to_element(term_guid, body=standard_comment)
    reply_response = fm_client.add_comment_reply(
        term_guid,
        comment_response["guid"],
        body=reply_comment,
        access_service_url_marker="asset-manager",
    )
    fm_client.remove_note_log(reply_response["guid"])
    fm_client.remove_note_log(comment_response["guid"])
    assert reply_response["relatedHTTPCode"] == 200
    assert reply_response["class"] == "GUIDResponse"
    assert "guid" in reply_response


#
## test_add_comment_to_element test
#
def test_add_comment_to_element():

    try:
        response = fm_client.add_comment_to_element(term_guid, body=standard_comment)
        fm_client.remove_note_log(response["guid"])
        assert response["relatedHTTPCode"] == 200
        assert response["class"] == "GUIDResponse"
        assert "guid" in response
    except PyegeriaException as e:
        print_basic_exception(e)
    except InvalidParameterException as e:
        print_exception_response(e)


def test_add_comment_to_element_live():
    guid = "d52a42e9-87a1-4382-aa0e-d0a3a63465f6"
    comment = "My simple command interface for Egeria"
    comment_type = "STANDARD_COMMENT"
    response = fm_client.add_comment_to_element(guid, comment, comment_type)
    print(response)


#
## test_add_like_to_element test
#
def test_add_like_to_element():
    response = fm_client.add_like_to_element(term_guid)
    fm_client.remove_like_from_element(term_guid)
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "VoidResponse"


#
## test_add_rating_to_element test
#
def test_add_rating_to_element():
    response = fm_client.add_rating_to_element(term_guid, body=rating)
    fm_client.remove_rating_from_element(term_guid)
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "VoidResponse"


#
## test_add_tag_to_element test
#
def test_add_tag_to_element():
    informal_tag = fm_client.create_informal_tag(tag_for_testing)
    response = fm_client.add_tag_to_element(term_guid, informal_tag["guid"])
    fm_client.delete_tag(informal_tag["guid"])
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "VoidResponse"


#
## test_clear_accepted_answer test
#


def test_clear_accepted_answer():
    question_comment_response = fm_client.add_comment_to_element(term_guid, body=question_comment)
    answer_comment_response = fm_client.add_comment_reply(
        term_guid, question_comment_response["guid"], body=answer_comment
    )
    setup_response = fm_client.setup_accepted_answer(
        question_comment_response["guid"], answer_comment_response["guid"]
    )
    response = fm_client.clear_accepted_answer(
        question_comment_response["guid"], answer_comment_response["guid"]
    )
    fm_client.remove_note_log(answer_comment_response["guid"])
    fm_client.remove_note_log(question_comment_response["guid"])
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "VoidResponse"


#
## test_create_informal_tag test
#
def test_create_informal_tag():
    my_test_tag_body = {
        "isPrivateTag": False,
        "name": "test-tag-from-python",
        "description": "this tag was created using the python API for testing purposes",
    }
    response = fm_client.create_informal_tag(my_test_tag_body)
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "GUIDResponse"
    assert valid_guid(response["guid"])
    fm_client.delete_tag(response["guid"])


#
## test_create_note test
#
def test_create_note():
    note_log_response = fm_client.create_note_log(term_guid, body=notelog_for_testing)
    response = fm_client.create_note(note_log_response["guid"], body=note_for_testing)
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "GUIDResponse"
    assert valid_guid(response["guid"])
    fm_client.remove_note_log(note_log_response["guid"])


#
## test_create_note_log test
#
def test_create_note_log():
    response = fm_client.create_note_log(term_guid, body=notelog_for_testing)
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "GUIDResponse"
    assert valid_guid(response["guid"])
    fm_client.remove_note_log(response["guid"])


#
## test_delete_tag test
#
def test_delete_tag():
    my_test_tag_body = {
        "isPrivateTag": False,
        "name": "test-tag-from-python",
        "description": "this tag was created using the python API for testing purposes",
    }
    create_response = fm_client.create_informal_tag(my_test_tag_body)
    assert create_response["relatedHTTPCode"] == 200
    assert create_response["class"] == "GUIDResponse"
    assert valid_guid(create_response["guid"])
    delete_response = fm_client.delete_tag(create_response["guid"])
    assert delete_response["relatedHTTPCode"] == 200
    assert delete_response["class"] == "VoidResponse"


#
## test_find_my_tags test
#
def test_find_my_tags():
    my_test_tag_body = {
        "isPrivateTag": True,
        "name": "one of my tags",
        "description": "this tag was created using the python API for testing purposes",
    }
    create_response = fm_client.create_informal_tag(my_test_tag_body)
    response = fm_client.find_my_tags({"filter": "one"}, starts_with=True)
    assert response[0]["isPrivateTag"] == my_test_tag_body["isPrivateTag"]
    assert response[0]["name"] == my_test_tag_body["name"]
    assert response[0]["description"] == my_test_tag_body["description"]
    delete_response = fm_client.delete_tag(create_response["guid"])


def test_find_my_tags_with_no_tags():
    response = fm_client.find_my_tags({"filter": "one"}, starts_with=True)
    assert response["class"] == "InformalTagsResponse"
    assert response["relatedHTTPCode"] == 200


def test_find_my_tags_with_no_tags_and_details():
    response = fm_client.find_my_tags(
        {"filter": "one"}, starts_with=True, detailed_response=True
    )
    assert response["class"] == "InformalTagsResponse"
    assert response["relatedHTTPCode"] == 200


#
## test_find_note_logs test
#
def test_find_note_logs():
    create_response = fm_client.create_note_log(term_guid, body=notelog_for_testing)
    response = fm_client.find_note_logs({"filter": ""})
    assert "class" in response[0]
    assert "qualifiedName" in response[0]
    assert "description" in response[0]
    assert "guid" in response[0]
    fm_client.remove_note_log(create_response["guid"])


def test_find_note_logs_detailed():
    create_response = fm_client.create_note_log(term_guid, body=notelog_for_testing)
    response = fm_client.find_note_logs({"filter": ""}, detailed_response=True)
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "NoteLogsResponse"
    assert "elementList" in response
    fm_client.remove_note_log(create_response["guid"])


def test_find_note_logs_with_no_note_logs():
    response = fm_client.find_note_logs({"filter": ""}, starts_with=True)
    assert response["class"] == "NoteLogsResponse"
    assert response["relatedHTTPCode"] == 200


def test_find_note_logs_with_no_no_note_and_details():
    response = fm_client.find_note_logs(
        {"filter": ""}, starts_with=True, detailed_response=True
    )
    assert response["class"] == "NoteLogsResponse"
    assert response["relatedHTTPCode"] == 200


#
## test_find_notes test
#
def test_find_notes():
    note_log_response = fm_client.create_note_log(term_guid, body=notelog_for_testing)
    response = fm_client.create_note(note_log_response["guid"], body=note_for_testing)
    response = fm_client.find_notes(body={"filter": ""}, output_format="JSON")
    assert "class" in response[0]
    assert "qualifiedName" in response[0]
    assert "guid" in response[0]
    fm_client.remove_note_log(note_log_response["guid"])


def test_find_notes_detailed():
    note_log_response = fm_client.create_note_log(term_guid, body=notelog_for_testing)
    note_response = fm_client.create_note(
        note_log_response["guid"], body=note_for_testing
    )
    response = fm_client.find_notes(body={"filter": ""}, output_format="JSON")
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "NotesResponse"
    assert "elementList" in response
    fm_client.remove_note_log(note_log_response["guid"])


def test_find_notes_with_no_notes():
    response = fm_client.find_notes(body={"filter": ""}, starts_with=True, output_format="JSON")
    assert response["class"] == "NotesResponse"
    assert response["relatedHTTPCode"] == 200


def test_find_notes_with_no_notes_and_details():
    response = fm_client.find_notes(body={"filter": ""}, starts_with=True, output_format="JSON")
    assert response["class"] == "NotesResponse"
    assert response["relatedHTTPCode"] == 200


#
## test_find_tags test
#
def test_find_tags():
    my_test_tag_body = {
        "isPrivateTag": False,
        "name": "This tag is public",
        "description": "this tag was created using the python API for testing purposes",
    }
    create_response = fm_client.create_informal_tag(my_test_tag_body)
    response = fm_client.find_tags({"filter": "public"}, ends_with=True)
    assert response[0]["isPrivateTag"] == my_test_tag_body["isPrivateTag"]
    assert response[0]["name"] == my_test_tag_body["name"]
    assert response[0]["description"] == my_test_tag_body["description"]
    delete_response = fm_client.delete_tag(create_response["guid"])


def test_find_tags_with_no_tags():
    response = fm_client.find_tags({"filter": "one"}, starts_with=True)
    assert response["class"] == "InformalTagsResponse"
    assert response["relatedHTTPCode"] == 200


def test_find_tags_with_no_tags_and_details():
    response = fm_client.find_tags({"filter": "one"}, starts_with=True)
    assert response["class"] == "InformalTagsResponse"
    assert response["relatedHTTPCode"] == 200


#
## test_find_comments test
#
def test_can_handle_no_comments():
    response = fm_client.find_note_logs({"filter": ""})
    assert response["class"] == "CommentElementsResponse"
    assert response["relatedHTTPCode"] == 200


def test_can_handle_no_comments_with_details_requested():
    response = fm_client.find_note_logs({"filter": ""})
    assert response["class"] == "CommentElementsResponse"
    assert response["relatedHTTPCode"] == 200


def test_find_comments():
    # create_response = fm_client.add_comment_to_element(term_guid, body=standard_comment)
    response = fm_client.find_note_logs("kafka")
    assert "class" in response[0]
    assert response[0]["properties"]["class"] == "CommentProperties"
    assert "qualifiedName" in response[0]["properties"].keys()
    assert "guid" in response[0]['elementHeader']
    # fm_client.remove_comment_from_element(create_response["guid"])

def test_get_comment_by_guid_live():
    try:
        response = fm_client.get_comment_by_guid('783c1136-df57-44f1-8c48-856cd871336d')
        print(json.dumps(response, indent=2))
    except PyegeriaException as e:
        print_basic_exception(e)
    except ValidationError as e:
        print_validation_error(e)

def test_find_comments_detailed():
    create_response = fm_client.add_comment_to_element(term_guid, body=standard_comment)
    response = fm_client.find_note_logs({"filter": ""})
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "CommentElementsResponse"
    assert "elementList" in response
    fm_client.remove_note_log(create_response["guid"])


#
## test_get_attached_comments test
#
def test_get_attached_comments():
    create_response = fm_client.add_comment_to_element(term_guid, body=standard_comment)
    response = fm_client.get_attached_note_logs(term_guid)
    assert "class" in response[0]
    assert response[0]["class"] == "CommentProperties"
    assert "qualifiedName" in response[0]
    assert "guid" in response[0]
    fm_client.remove_note_log(create_response["guid"])


def test_get_attached_comments_detailed():
    create_response = fm_client.add_comment_to_element(term_guid, body=standard_comment)
    response = fm_client.get_attached_note_logs(term_guid)
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "CommentElementsResponse"
    assert "elementList" in response
    fm_client.remove_note_log(create_response["guid"])


def test_get_attached_comments_with_no_comments():
    response = fm_client.get_attached_note_logs(term_guid)
    assert response["class"] == "CommentElementsResponse"
    assert response["relatedHTTPCode"] == 200


def test_get_attached_comments_with_no_comments_and_details():
    response = fm_client.get_attached_note_logs(term_guid)
    assert response["class"] == "CommentElementsResponse"
    assert response["relatedHTTPCode"] == 200


#
## test_get_attached_likes test
#
def test_get_attached_likes():
    create_response = fm_client.add_like_to_element(term_guid)
    response = fm_client.get_attached_likes(term_guid)
    assert "user" in response[0]
    assert "guid" in response[0]
    fm_client.remove_like_from_element(term_guid)


def test_get_attached_likes_detailed():
    create_response = fm_client.add_like_to_element(term_guid)
    response = fm_client.get_attached_likes(term_guid, detailed_response=True)
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "LikeElementsResponse"
    assert "elementList" in response
    fm_client.remove_like_from_element(term_guid)


def test_get_attached_likes_with_no_likes():
    response = fm_client.get_attached_likes(term_guid)
    assert response["class"] == "LikeElementsResponse"
    assert response["relatedHTTPCode"] == 200


def test_get_attached_likes_with_no_likes_and_details():
    response = fm_client.get_attached_likes(term_guid, detailed_response=True)
    assert response["class"] == "LikeElementsResponse"
    assert response["relatedHTTPCode"] == 200


#
## test_get_attached_ratings test
#


def test_get_attached_ratings():
    create_response = fm_client.add_rating_to_element(term_guid, body=rating)
    response = fm_client.get_attached_ratings(term_guid)
    assert "starRating" in response[0]
    assert "guid" in response[0]
    fm_client.remove_rating_from_element(term_guid)


def test_get_attached_ratings_detailed():
    create_response = fm_client.add_rating_to_element(term_guid, body=rating)
    response = fm_client.get_attached_ratings(term_guid, detailed_response=True)
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "RatingElementsResponse"
    assert "elementList" in response
    fm_client.remove_rating_from_element(term_guid)


def test_get_attached_ratings_with_no_ratings():
    response = fm_client.get_attached_ratings(term_guid)
    assert response["class"] == "RatingElementsResponse"
    assert response["relatedHTTPCode"] == 200


def test_get_attached_ratings_with_no_ratings_and_details():
    response = fm_client.get_attached_ratings(term_guid, detailed_response=True)
    assert response["class"] == "RatingElementsResponse"
    assert response["relatedHTTPCode"] == 200


#
## test_get_attached_tags test
#
def test_get_attached_tags():
    create_response = fm_client.create_informal_tag(tag_for_testing)
    fm_client.add_tag_to_element(term_guid, create_response["guid"])
    response = fm_client.get_attached_tags(term_guid)
    assert "name" in response[0]
    assert "isPrivateTag" in response[0]
    assert "guid" in response[0]
    fm_client.delete_tag(create_response["guid"])


def test_get_attached_tags_detailed():
    create_response = fm_client.create_informal_tag(tag_for_testing)
    fm_client.add_tag_to_element(term_guid, create_response["guid"])
    response = fm_client.get_attached_tags(term_guid, detailed_response=True)
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "InformalTagsResponse"
    assert "tags" in response
    fm_client.delete_tag(create_response["guid"])


def test_get_attached_tags_with_no_tags():
    response = fm_client.get_attached_tags(term_guid)
    assert response["class"] == "InformalTagsResponse"
    assert response["relatedHTTPCode"] == 200


def test_get_attached_tags_with_no_tags_and_details():
    response = fm_client.get_attached_tags(term_guid, detailed_response=True)
    assert response["class"] == "InformalTagsResponse"
    assert response["relatedHTTPCode"] == 200


#
## test_get_comment test
#


def test_get_comment():
    create_response = fm_client.add_comment_to_element(term_guid, body=standard_comment)
    response = fm_client.get_comment(create_response["guid"])
    assert "class" in response
    assert response["class"] == "CommentProperties"
    assert "qualifiedName" in response
    assert "guid" in response
    fm_client.remove_note_log(create_response["guid"])


def test_get_comment_detailed():
    create_response = fm_client.add_comment_to_element(term_guid, body=standard_comment)
    response = fm_client.get_comment(create_response["guid"], detailed_response=True)
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "CommentResponse"
    fm_client.remove_note_log(create_response["guid"])


def test_get_comment_with_bad_guid():
    with pytest.raises(InvalidParameterException):
        response = fm_client.get_comment("1234")


#
## test_get_elements_by_tag test
#
def test_get_elements_by_tag():
    create_response = fm_client.create_informal_tag(tag_for_testing)
    fm_client.add_tag_to_element(term_guid, create_response["guid"])
    response = fm_client.get_search_keyword_by_guid(create_response["guid"], JSON)
    assert "name" in response[0]
    assert "typeName" in response[0]
    assert "properties" in response[0]
    assert "guid" in response[0]
    fm_client.delete_tag(create_response["guid"])


def test_get_elements_by_tag_detailed():
    create_response = fm_client.create_informal_tag(tag_for_testing)
    fm_client.add_tag_to_element(term_guid, create_response["guid"])
    response = fm_client.get_search_keyword_by_guid(create_response["guid"], JSON)
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "RelatedElementsResponse"
    assert "elementList" in response
    fm_client.delete_tag(create_response["guid"])


#
## test_get_note_by_guid test
#


def test_get_note_by_guid():
    note_log_response = fm_client.create_note_log(term_guid, body=notelog_for_testing)
    note_response = fm_client.create_note(
        note_log_response["guid"], body=note_for_testing
    )
    response = fm_client.get_note_by_guid(note_response["guid"], output_format="JSON")
    assert "class" in response
    assert "qualifiedName" in response
    assert "guid" in response
    fm_client.remove_note_log(note_log_response["guid"])


def test_get_note_by_guid_detailed():
    note_log_response = fm_client.create_note_log(term_guid, body=notelog_for_testing)
    note_response = fm_client.create_note(
        note_log_response["guid"], body=note_for_testing
    )
    response = fm_client.get_note_by_guid(note_response["guid"], output_format="JSON")
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "NoteResponse"
    assert "element" in response
    fm_client.remove_note_log(note_log_response["guid"])


#
## test_get_note_log_by_guid test
#


def test_get_note_log_by_guid():
    note_log_response = fm_client.create_note_log(term_guid, body=notelog_for_testing)
    response = fm_client.get_note_log_by_guid(note_log_response["guid"])
    assert "class" in response
    assert response["class"] == "NoteLogProperties"
    assert "qualifiedName" in response
    assert "guid" in response
    fm_client.remove_note_log(note_log_response["guid"])


def test_get_note_log_by_guid_detailed():
    note_log_response = fm_client.create_note_log(term_guid, body=notelog_for_testing)
    response = fm_client.get_note_log_by_guid(
        note_log_response["guid"], detailed_response=True
    )
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "NoteLogResponse"
    assert "element" in response
    fm_client.remove_note_log(note_log_response["guid"])


#
## test_get_note_logs_by_name test
#


def test_get_note_logs_by_name():
    note_log_response = fm_client.create_note_log(term_guid, body=notelog_for_testing)
    response = fm_client.get_note_logs_by_name(
        {"filter": notelog_for_testing["qualifiedName"]}
    )
    assert "class" in response[0]
    assert response[0]["class"] == "NoteLogProperties"
    assert "qualifiedName" in response[0]
    assert "guid" in response[0]
    fm_client.remove_note_log(note_log_response["guid"])


def test_get_note_logs_by_name_detailed():
    note_log_response = fm_client.create_note_log(term_guid, body=notelog_for_testing)
    response = fm_client.get_note_logs_by_name(
        {"filter": notelog_for_testing["qualifiedName"]}, detailed_response=True
    )
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "NoteLogsResponse"
    assert "elementList" in response
    fm_client.remove_note_log(note_log_response["guid"])


#
## test_get_note_logs_for_element test
#


def test_get_note_logs_for_element():
    note_log_response = fm_client.create_note_log(term_guid, body=notelog_for_testing)
    response = fm_client.get_note_logs_for_element(term_guid)
    assert "class" in response[0]
    assert response[0]["class"] == "NoteLogProperties"
    assert "qualifiedName" in response[0]
    assert "guid" in response[0]
    fm_client.remove_note_log(note_log_response["guid"])


def test_get_note_logs_for_element_detailed():
    note_log_response = fm_client.create_note_log(term_guid, body=notelog_for_testing)
    response = fm_client.get_note_logs_for_element(term_guid, detailed_response=True)
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "NoteLogsResponse"
    assert "elementList" in response
    fm_client.remove_note_log(note_log_response["guid"])


#
## test_get_notes_for_note_log test
#
def test_get_notes_for_note_log():
    note_log_response = fm_client.create_note_log(term_guid, body=notelog_for_testing)
    response = fm_client.create_note(note_log_response["guid"], body=note_for_testing)
    response = fm_client.get_notes_for_note_log(note_log_response["guid"])
    assert "class" in response[0]
    assert "qualifiedName" in response[0]
    assert "guid" in response[0]
    fm_client.remove_note_log(note_log_response["guid"])


def test_get_notes_for_note_log_detailed():
    note_log_response = fm_client.create_note_log(term_guid, body=notelog_for_testing)
    note_response = fm_client.create_note(
        note_log_response["guid"], body=note_for_testing
    )
    response = fm_client.get_notes_for_note_log(
        note_log_response["guid"], detailed_response=True
    )
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "NotesResponse"
    assert "elementList" in response
    fm_client.remove_note_log(note_log_response["guid"])


#
## test_get_tag test
#
def test_get_tag():
    my_test_tag_body = {
        "isPrivateTag": False,
        "name": "test-tag-from-python",
        "description": "this tag was created using the python API for testing purposes",
    }
    create_response = fm_client.create_informal_tag(my_test_tag_body)
    response = fm_client.get_tag(create_response["guid"])
    assert response["isPrivateTag"] == my_test_tag_body["isPrivateTag"]
    assert response["name"] == my_test_tag_body["name"]
    assert response["description"] == my_test_tag_body["description"]
    delete_response = fm_client.delete_tag(create_response["guid"])


def test_get_tags_by_name():
    response = fm_client.get_tags_by_name("Meow")
    print(json.dumps(response, indent=2))

#
## test_get_tags_by_name test 1
#
def test_get_tags_by_name_test1():
    my_test_tag_body = {
        "isPrivateTag": False,
        "name": "Meow",
        "description": "this tag was created using the python API for testing purposes",
    }
    create_response = fm_client.get_tags_by_name(my_test_tag_body)
    response = fm_client.get_tags_by_name({"filter": my_test_tag_body["name"]})
    assert response[0]["isPrivateTag"] == my_test_tag_body["isPrivateTag"]
    assert response[0]["name"] == my_test_tag_body["name"]
    assert response[0]["description"] == my_test_tag_body["description"]
    delete_response = fm_client.delete_tag(create_response["guid"])


#
## test_get_tags_by_name test 2
#
def test_get_tags_by_name_test2():
    my_test_tag_body = {
        "isPrivateTag": False,
        "name": "test-tag-from-python",
        "description": "this tag was created using the python API for testing purposes",
    }
    create_1_response = fm_client.create_informal_tag(my_test_tag_body)
    create_2_response = fm_client.create_informal_tag(my_test_tag_body)
    response = fm_client.get_tags_by_name({"filter": my_test_tag_body["name"]})
    assert len(response) >= 2
    assert response[0]["isPrivateTag"] == my_test_tag_body["isPrivateTag"]
    assert response[0]["name"] == my_test_tag_body["name"]
    assert response[0]["description"] == my_test_tag_body["description"]
    assert response[1]["isPrivateTag"] == my_test_tag_body["isPrivateTag"]
    assert response[1]["name"] == my_test_tag_body["name"]
    assert response[1]["description"] == my_test_tag_body["description"]
    fm_client.delete_tag(create_1_response["guid"])
    fm_client.delete_tag(create_2_response["guid"])


#
## test_get_tags_by_name test 3
#
def test_get_tags_by_name_test3():
    my_test_tag_body = {
        "isPrivateTag": False,
        "name": "test-tag-from-python",
        "description": "this tag was created using the python API for testing purposes",
    }
    create_response = fm_client.create_informal_tag(my_test_tag_body)
    #    response = fm_client.get_tags_by_name(my_test_tag_body['name'], "view-server", 0, 10, True)
    response = fm_client.get_tags_by_name(
        {"filter": my_test_tag_body["name"]}, detailed_response=True
    )
    assert response["class"] == "InformalTagsResponse"
    assert response["relatedHTTPCode"] == 200
    assert "elementHeader" in response["tags"][0]
    assert "properties" in response["tags"][0]
    delete_response = fm_client.delete_tag(create_response["guid"])


#
## test_remove_comment_from_element test
#
def test_remove_comment_from_element():
    create_comment = fm_client.add_comment_to_element(term_guid, body=standard_comment)
    response = fm_client.remove_note_log(create_comment["guid"])
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "VoidResponse"


#
## test_remove_like_from_element test
#
def test_remove_like_from_element():
    fm_client.add_like_to_element(term_guid)
    response = fm_client.remove_like_from_element(term_guid)
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "VoidResponse"


#
## test_remove_note test
#
def test_remove_note():
    note_log_response = fm_client.create_note_log(term_guid, body=notelog_for_testing)
    note_response = fm_client.create_note(
        note_log_response["guid"], body=note_for_testing
    )
    response = fm_client.remove_note_log(note_log_response["guid"])
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "VoidResponse"
    #


## test_remove_note_log test
#
def test_remove_note_log():
    create_response = fm_client.create_note_log(term_guid, body=notelog_for_testing)
    response = fm_client.remove_note_log(create_response["guid"])
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "VoidResponse"


#
## test_remove_rating_from_element test
#
def test_remove_rating_from_element():
    fm_client.add_rating_to_element(term_guid, body=rating)
    response = fm_client.remove_rating_from_element(term_guid)
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "VoidResponse"


#
## test_remove_tag_from_element test
#
def test_remove_tag_from_element():
    informal_tag = fm_client.create_informal_tag(tag_for_testing)
    fm_client.add_tag_to_element(term_guid, informal_tag["guid"])
    response = fm_client.remove_tag_from_element(term_guid, informal_tag["guid"])
    fm_client.delete_tag(informal_tag["guid"])
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "VoidResponse"


#
## test_setup_accepted_answer test
#
def test_setup_accepted_answer():
    question_comment_response = fm_client.add_comment_to_element(term_guid, body=question_comment)
    answer_comment_response = fm_client.add_comment_reply(
        term_guid, question_comment_response["guid"], body=answer_comment
    )
    response = fm_client.setup_accepted_answer(
        question_comment_response["guid"], answer_comment_response["guid"]
    )
    fm_client.remove_note_log(answer_comment_response["guid"])
    fm_client.remove_note_log(question_comment_response["guid"])
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "VoidResponse"


#
## test_update_comment test
#
def test_update_comment():
    add_response = fm_client.add_comment_to_element(term_guid, body=standard_comment)
    response = fm_client.update_comment(add_response["guid"], updated_standard_comment)
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "VoidResponse"
    fm_client.remove_note_log(add_response["guid"])


#
## test_update_comment_visibility test
#
def test_update_comment_visibility():
    add_response = fm_client.add_comment_to_element(term_guid, body=standard_comment)
    response = fm_client.update_comment_visibility(
        term_guid, add_response["guid"], is_public=True
    )
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "VoidResponse"
    fm_client.remove_note_log(add_response["guid"])


def test_update_comment_visibility2():
    add_response = fm_client.add_comment_to_element(term_guid, body=standard_comment)
    response = fm_client.update_comment_visibility(
        term_guid, add_response["guid"], is_public=False
    )
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "VoidResponse"
    fm_client.remove_note_log(add_response["guid"])


#
## test_update_note test
#
def test_update_note():
    note_log_response = fm_client.create_note_log(term_guid, body=notelog_for_testing)
    jprint(note_log_response)
    note_response = fm_client.create_note(
        note_log_response["guid"], body=note_for_testing
    )
    jprint(note_response, "Note Response:")
    jprint(updated_note_for_testing, "Updated Note for testing:")
    response = fm_client.update_note(note_response["guid"], updated_note_for_testing)
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "VoidResponse"
    fm_client.remove_note_log(note_log_response["guid"])


#
## test_update_note_log test
#
def test_update_note_log():
    create_response = fm_client.create_note_log(term_guid, body=notelog_for_testing)
    response = fm_client.update_note_log(
        create_response["guid"], updated_notelog_for_testing, is_merge_update=True
    )
    assert response["relatedHTTPCode"] == 200
    assert response["class"] == "VoidResponse"
    fm_client.remove_note_log(create_response["guid"])


#
## test_update_tag_description test
#
def test_update_tag_description():
    my_test_tag_body = {
        "isPrivateTag": False,
        "name": "test-tag-from-python",
        "description": "this tag was created using the python API for testing purposes",
    }
    create_response = fm_client.create_informal_tag(my_test_tag_body)
    updated_description = "This is my updated description"
    update_response = fm_client.update_tag_description(
        create_response["guid"], {"description": updated_description}
    )
    assert update_response["relatedHTTPCode"] == 200
    assert update_response["class"] == "VoidResponse"
    get_response = fm_client.get_tag(create_response["guid"])
    assert get_response["description"] == updated_description
    fm_client.delete_tag(create_response["guid"])


def test_find_comments():
    search_string = "*"
    response = fm_client.find_comments(search_string, output_format="DICT")
    print(json.dumps(response, indent=2))

def test_add_comment_to_element():
    comment_guid = "1d463812-3dab-4dd6-84b0-1378ed899912"
    element_guid = "fc98766a-d334-4ca0-90dd-d29d0013cddd"
    try:
        fm_client.add_comment_to_element(element_guid, "Meow" )
    except PyegeriaException as e:
        print_basic_exception(e)
# comment guid 1d463812-3dab-4dd6-84b0-1378ed899912

def test_remove_comment_from_element():
    comment_guid = "1d463812-3dab-4dd6-84b0-1378ed899912"
    element_guid = "fc98766a-d334-4ca0-90dd-d29d0013cddd"
    try:
        fm_client.remove_comment_from_element(element_guid, comment_guid)
    except PyegeriaException as e:
        print_basic_exception(e)

def test_create_tag():
    create_response = fm_client.create_informal_tag("Puddy Approves","Puddy voices approval")
    print(create_response)
    assert create_response["relatedHTTPCode"] == 200
    assert create_response["class"] == "GUIDResponse"
    global tag_guid
    tag_guid = create_response["guid"]
    print(tag_guid)

def test_delete_tag():
    delete_response = fm_client.delete_tag(tag_guid)
    print(delete_response)

def test_find_tags():
    search_string = "*"
    response = fm_client.find_tags(search_string)
    print(json.dumps(response, indent=2))




"""
This is an ongoing experiment in parsing and playing with Freddie docs
"""

import nbformat
import os
from rich import print
from rich.console import Console

from pyegeria.egeria_tech_client import EgeriaTech
from pyegeria.md_processing_utils import (extract_command, process_glossary_upsert_command, process_term_upsert_command,
                                          get_current_datetime_string, process_per_proj_upsert_command, command_list,
                                          )
import click

from pyegeria._exceptions import (
    print_exception_response,
)

EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get("KAFKA_ENDPOINT", "localhost:9092")
EGERIA_PLATFORM_URL = os.environ.get("EGERIA_PLATFORM_URL", "https://localhost:9443")
EGERIA_VIEW_SERVER = os.environ.get("EGERIA_VIEW_SERVER", "view-server")
EGERIA_VIEW_SERVER_URL = os.environ.get(
    "EGERIA_VIEW_SERVER_URL", "https://localhost:9443"
)
EGERIA_INTEGRATION_DAEMON = os.environ.get("EGERIA_INTEGRATION_DAEMON", "integration-daemon")
EGERIA_INTEGRATION_DAEMON_URL = os.environ.get(
    "EGERIA_INTEGRATION_DAEMON_URL", "https://localhost:9443"
)
EGERIA_ADMIN_USER = os.environ.get("ADMIN_USER", "garygeeke")
EGERIA_ADMIN_PASSWORD = os.environ.get("ADMIN_PASSWORD", "secret")
EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")
EGERIA_WIDTH = os.environ.get("EGERIA_WIDTH", 220)
EGERIA_JUPYTER = os.environ.get("EGERIA_JUPYTER", False)
EGERIA_HOME_GLOSSARY_GUID = os.environ.get("EGERIA_HOME_GLOSSARY_GUID", None)
EGERIA_GLOSSARY_PATH = os.environ.get("EGERIA_GLOSSARY_PATH", None)
EGERIA_ROOT_PATH = os.environ.get("EGERIA_ROOT_PATH", "../../")
EGERIA_INBOX_PATH = os.environ.get("EGERIA_INBOX_PATH", "md_processing/dr_egeria_inbox")
EGERIA_OUTBOX_PATH = os.environ.get("EGERIA_OUTBOX_PATH", "md_processing/dr_egeria_outbox")

console = Console(width=int(EGERIA_WIDTH))



@click.command("process-jupyter")
@click.option("--file-path", help="File path to notebook",
              default="glossary_creation_experiment.ipynb")
@click.option("--directive", default="display-only", help="How to process the file")
@click.option("--server", default=EGERIA_VIEW_SERVER, help="Egeria view server to use.")
@click.option(
    "--url", default=EGERIA_VIEW_SERVER_URL, help="URL of Egeria platform to connect to"
)
@click.option("--userid", default=EGERIA_USER, help="Egeria user")
@click.option("--user_pass", default=EGERIA_USER_PASSWORD, help="Egeria user password")
def process_jupyter_notebook(
        file_path: str,
        directive: str,
        server: str,
        url: str,
        userid: str,
        user_pass: str,
        ):
    client = EgeriaTech(server, url, user_id=userid)
    token = client.create_egeria_bearer_token(userid, user_pass)

    element_dictionary = {}

    try:
        updated = False
        full_file_path = os.path.join(EGERIA_ROOT_PATH, EGERIA_INBOX_PATH, file_path)
        click.echo(f"Processing notebook: {full_file_path}")
        with open(full_file_path, 'r') as f:
            nb = nbformat.read(f, as_version=4)
            for cell in nb.cells:
                if cell.cell_type == 'markdown':
                    if cell.source.strip().startswith('#'):
                        potential_command = extract_command(cell.source)
                        if potential_command in command_list:
                            if potential_command in ["Create Glossary", "Update Glossary"]:
                                result = process_glossary_upsert_command(client, cell.source, directive)

                            elif potential_command in ["Create Term", "Update Term"]:
                                result = process_term_upsert_command(client, cell.source, directive)
                            elif potential_command in ["Create Personal Project", "Update Personal Project"]:
                                result = process_per_proj_upsert_command(client, cell.source, directive)
                            else:
                                # If object_action is not recognized, copy the block as-is
                                result = None

                            if result:
                                if directive == "process":
                                    updated = True
                                    cell.source = result
                                    # print(json.dumps(element_dictionary, indent=4))
                            elif directive == "process":
                                # Handle case with errors (skip this block but notify the user)
                                print(f"\n==>\tErrors found while processing command: \'{potential_command}\'\n"
                                      f"\tPlease correct and try again. \n")


        if updated:
            path, filename = os.path.split(file_path)  # Get both parts
            new_filename = f"processed-{get_current_datetime_string()}-{filename}"  # Create the new filename
            new_file_path = os.path.join(EGERIA_ROOT_PATH, EGERIA_OUTBOX_PATH, new_filename)  # Construct the new path
            os.makedirs(os.path.dirname(new_file_path), exist_ok=True)


            with open(new_file_path, 'w') as f2:
                nbformat.write(nb, f2)
            click.echo(f"\n==>Notebook written to {new_file_path}")
        else:
            click.echo("\nNo updates detected. New file not created.")

    except Exception as e:
        print_exception_response(e)
        return


if __name__ == "__main__":
    process_jupyter_notebook()
